@@ -12,6 +12,9 @@ GType anjuta_shell_error_get_type (void) G_GNUC_CONST;
 #define ANJUTA_TYPE_SHELL_ERROR (anjuta_shell_error_get_type())
 GType anjuta_shell_placement_get_type (void) G_GNUC_CONST;
 #define ANJUTA_TYPE_SHELL_PLACEMENT (anjuta_shell_placement_get_type())
+/* enumerations from "anjuta-convert.h" */
+GType gedit_convert_error_get_type (void) G_GNUC_CONST;
+#define ANJUTA_TYPE_CONVERT_ERROR (gedit_convert_error_get_type())
 /* enumerations from "anjuta-plugin-description.h" */
 GType anjuta_plugin_description_parse_error_get_type (void) G_GNUC_CONST;
 #define ANJUTA_TYPE_PLUGIN_DESCRIPTION_PARSE_ERROR (anjuta_plugin_description_parse_error_get_type())
