@@ -84,6 +84,7 @@ void anjuta_util_glist_strings_free(GList* list);
 void anjuta_util_glist_strings_prefix (GList * list, const gchar *prefix);
 void anjuta_util_glist_strings_sufix (GList * list, const gchar *sufix);
 GList* anjuta_util_glist_strings_sort (GList * list);
+gchar* anjuta_util_glist_strings_join (GList * list, gchar *delimiter);
 
 /**********************************************************/
 /* Both the returned glist and the data should be g_freed */
@@ -108,9 +109,22 @@ pid_t anjuta_util_execute_shell (const gchar *dir, const gchar *command);
 gchar* anjuta_util_escape_quotes(const gchar* str);
 
 gboolean anjuta_util_path_has_extension (const gchar *path, const gchar *ext);
+gchar* anjuta_util_get_real_path (const gchar *path);
+
+gchar* anjuta_util_uri_get_dirname (const gchar *uri);
+gchar* anjuta_util_replace_home_dir_with_tilde (const gchar *uri);
+gchar* anjuta_util_str_middle_truncate (const gchar *string,
+										 guint        truncate_length);
 
 gchar* anjuta_util_get_uri_mime_type (const gchar *uri);
 
+void anjuta_util_help_display (GtkWindow   *parent,
+							   const gchar *doc_id,
+							   const gchar *file_name);
+
+gchar *anjuta_util_get_user_config_dir (void);
+
+
 /* Temporarily copied here */
 
 #define ANJUTA_TYPE_BEGIN(class_name, prefix, parent_type) \
