@@ -14,13 +14,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
 #include <gtk/gtkactiongroup.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libanjuta/anjuta-preferences.h>
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-wizard.h>
@@ -128,7 +128,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 static void
 dispose (GObject *obj)
 {
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void
@@ -143,7 +143,7 @@ finalize (GObject *obj)
 	if(cg_plugin->generator != NULL)
 		g_object_unref(G_OBJECT(cg_plugin->generator));
 
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -198,17 +198,13 @@ cg_plugin_add_to_project (AnjutaClassGenPlugin *plugin,
 
 	if (dirname != NULL && strcmp (dirname, ".") != 0)
 	{
-		added_files = ianjuta_project_manager_add_source_multi (manager,
-		                                                        filenames,
-		                                                        dirname,
-		                                                        NULL);
+		added_files = ianjuta_project_manager_add_sources (manager, filenames,
+														   dirname, NULL);
 	}
 	else
 	{
-		added_files = ianjuta_project_manager_add_source_multi (manager,
-		                                                        filenames,
-		                                                        curdir,
-		                                                        NULL);
+		added_files = ianjuta_project_manager_add_sources (manager, filenames,
+														   curdir, NULL);
 	}
 
 	if (g_list_length (added_files) != 2)
@@ -452,7 +448,9 @@ iwizard_activate (IAnjutaWizard *wiz, G_GNUC_UNUSED GError **err)
 	AnjutaClassGenPlugin *cg_plugin;
 	gchar *user_name;
 	gchar *user_email;
-
+	IAnjutaProjectManagerCapabilities caps =
+		IANJUTA_PROJECT_MANAGER_CAN_ADD_NONE;
+	
 	cg_plugin = ANJUTA_PLUGIN_CLASS_GEN (wiz);
 
 	if (cg_plugin->window != NULL)
@@ -474,8 +472,17 @@ iwizard_activate (IAnjutaWizard *wiz, G_GNUC_UNUSED GError **err)
 	g_free(user_name);
 	g_free(user_email);
 
-	/* Check whether we have a loaded project or not */
-	if(cg_plugin->top_dir == NULL)
+	/* Check whether we have a loaded project and it can add sources */
+	if (cg_plugin->top_dir)
+	{
+		IAnjutaProjectManager *manager =
+			anjuta_shell_get_interface (ANJUTA_PLUGIN (wiz)->shell,
+										IAnjutaProjectManager, NULL);
+       if (manager)
+			caps = ianjuta_project_manager_get_capabilities (manager, NULL);
+	}
+
+	if((caps & IANJUTA_PROJECT_MANAGER_CAN_ADD_SOURCE) == FALSE)
 	{
 		cg_window_set_add_to_project (cg_plugin->window, FALSE);
 		cg_window_enable_add_to_project (cg_plugin->window, FALSE);
