@@ -30,38 +30,20 @@
 
 /*#define DEBUG*/
 #include <libanjuta/anjuta-debug.h>
-#include <libanjuta/interfaces/ianjuta-breakpoint-debugger.h>
-#include <libanjuta/interfaces/ianjuta-cpu-debugger.h>
-#include <libanjuta/interfaces/ianjuta-variable-debugger.h>
+#include <libanjuta/interfaces/ianjuta-debugger-breakpoint.h>
+#include <libanjuta/interfaces/ianjuta-debugger-register.h>
+#include <libanjuta/interfaces/ianjuta-debugger-memory.h>
+#include <libanjuta/interfaces/ianjuta-debugger-instruction.h>
+#include <libanjuta/interfaces/ianjuta-debugger-variable.h>
 
 #include <stdarg.h>
 
 /* Contants defintion
  *---------------------------------------------------------------------------*/
 
-#define STATE_TO_CHANGE	8  /* To convert IAnjutaDebuggerStatus to command go to */
-#define STATE_TO_NEED 16   /* To convert IAnjutaDebuggerStatus to command need */
-
 /* Private type
  *---------------------------------------------------------------------------*/
 
-enum
-{
-	NO_CHANGE = 0,
-	COMMAND_MASK = 0xff,
-	CHANGE_MASK = 0x3F << STATE_TO_CHANGE,
-	STOP_DEBUGGER = 1 << (IANJUTA_DEBUGGER_STOPPED + STATE_TO_CHANGE - 1),
-	START_DEBUGGER = 1 << (IANJUTA_DEBUGGER_STARTED + STATE_TO_CHANGE - 1),
-	LOAD_PROGRAM = 1 << (IANJUTA_DEBUGGER_PROGRAM_LOADED + STATE_TO_CHANGE - 1),
-	STOP_PROGRAM = 1 << (IANJUTA_DEBUGGER_PROGRAM_STOPPED + STATE_TO_CHANGE - 1),
-	RUN_PROGRAM = 1 << (IANJUTA_DEBUGGER_PROGRAM_RUNNING + STATE_TO_CHANGE - 1),
-	NEED_DEBUGGER_STOPPED = STOP_DEBUGGER << (STATE_TO_NEED - STATE_TO_CHANGE),
-	NEED_DEBUGGER_STARTED = START_DEBUGGER << (STATE_TO_NEED - STATE_TO_CHANGE),
-	NEED_PROGRAM_LOADED = LOAD_PROGRAM << (STATE_TO_NEED - STATE_TO_CHANGE),
-	NEED_PROGRAM_STOPPED = STOP_PROGRAM << (STATE_TO_NEED - STATE_TO_CHANGE),
-	NEED_PROGRAM_RUNNING = RUN_PROGRAM << (STATE_TO_NEED - STATE_TO_CHANGE)
-};
-
 typedef enum
 {
 	EMPTY_COMMAND ,
@@ -74,15 +56,18 @@ typedef enum
 	INSPECT_MEMORY_COMMAND,
 	DISASSEMBLE_COMMAND,
 	LIST_REGISTER_COMMAND,
+	SET_WORKING_DIRECTORY_COMMAND,
+	SET_ENVIRONMENT_COMMAND,
 	UNLOAD_COMMAND,         /* Program loaded */
 	START_COMMAND,         
 	BREAK_LINE_COMMAND,		  /* Program loaded - Program stopped */
 	BREAK_FUNCTION_COMMAND,
-	BREAK_ADDRESS_COMMAND,
+	BREAK_ADDRESS_COMMAND,	/* 0x10 */
 	ENABLE_BREAK_COMMAND,
-	IGNORE_BREAK_COMMAND,	/* 0x10 */
+	IGNORE_BREAK_COMMAND,
 	CONDITION_BREAK_COMMAND,
 	REMOVE_BREAK_COMMAND,
+	LIST_BREAK_COMMAND,
 	INFO_SHAREDLIB_COMMAND,
 	INFO_TARGET_COMMAND,
 	INFO_PROGRAM_COMMAND,
@@ -92,11 +77,14 @@ typedef enum
 	STEP_OUT_COMMAND,
 	RUN_COMMAND,		
 	RUN_TO_COMMAND,
+	STEPI_IN_COMMAND,
+	STEPI_OVER_COMMAND,			/* 0x20 */
+	RUN_TO_ADDRESS_COMMAND,
 	EXIT_COMMAND,
 	HANDLE_SIGNAL_COMMAND,
 	LIST_LOCAL_COMMAND,
 	LIST_ARG_COMMAND,
-	LIST_THREAD_COMMAND,		/* 0x20 */
+	LIST_THREAD_COMMAND,
 	SET_THREAD_COMMAND,
 	INFO_THREAD_COMMAND,
 	INFO_SIGNAL_COMMAND,
@@ -106,13 +94,13 @@ typedef enum
 	SET_FRAME_COMMAND,
 	LIST_FRAME_COMMAND,
 	UPDATE_REGISTER_COMMAND,
-	WRITE_REGISTER_COMMAND,
+	WRITE_REGISTER_COMMAND,		/* 0x30 */
 	EVALUATE_COMMAND,
 	INSPECT_COMMAND,
 	PRINT_COMMAND,
 	CREATE_VARIABLE,
 	EVALUATE_VARIABLE,
-	LIST_VARIABLE_CHILDREN,	/* 0x30 */
+	LIST_VARIABLE_CHILDREN,
 	DELETE_VARIABLE,
 	ASSIGN_VARIABLE,		
 	UPDATE_VARIABLE,
@@ -132,7 +120,7 @@ typedef enum
 		NEED_DEBUGGER_STOPPED | NEED_DEBUGGER_STARTED,
 	DMA_QUIT_COMMAND =
 		QUIT_COMMAND | CANCEL_ALL_COMMAND | STOP_DEBUGGER |
-	    NEED_DEBUGGER_STARTED | NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED,
+	    HIGH_PRIORITY | NEED_DEBUGGER_STARTED | NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED,
 	DMA_ABORT_COMMAND =
 		ABORT_COMMAND | CANCEL_ALL_COMMAND | STOP_DEBUGGER |
 	    ASYNCHRONOUS | NEED_DEBUGGER_STARTED | NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
@@ -148,6 +136,12 @@ typedef enum
 	DMA_LIST_REGISTER_COMMAND =
 		LIST_REGISTER_COMMAND |
 		NEED_DEBUGGER_STARTED | NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
+	DMA_SET_WORKING_DIRECTORY_COMMAND =
+		SET_WORKING_DIRECTORY_COMMAND |
+		NEED_DEBUGGER_STARTED | NEED_PROGRAM_LOADED,
+	DMA_SET_ENVIRONMENT_COMMAND =
+		SET_ENVIRONMENT_COMMAND |
+		NEED_DEBUGGER_STARTED | NEED_PROGRAM_LOADED,
 	DMA_UNLOAD_COMMAND =
 		UNLOAD_COMMAND | START_DEBUGGER |
 		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED,
@@ -156,25 +150,28 @@ typedef enum
 		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED,
 	DMA_BREAK_LINE_COMMAND =
 		BREAK_LINE_COMMAND |
-		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
+		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED,
 	DMA_BREAK_FUNCTION_COMMAND =
 		BREAK_FUNCTION_COMMAND |
-		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
+		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED,
 	DMA_BREAK_ADDRESS_COMMAND =
 		BREAK_ADDRESS_COMMAND |
-		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
+		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED,
 	DMA_ENABLE_BREAK_COMMAND =
 		ENABLE_BREAK_COMMAND |
-		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
+		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED,
 	DMA_IGNORE_BREAK_COMMAND =
 		IGNORE_BREAK_COMMAND |
-		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
+		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED,
 	DMA_CONDITION_BREAK_COMMAND =
 		CONDITION_BREAK_COMMAND |
-		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
+		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED,
 	DMA_REMOVE_BREAK_COMMAND =
 		REMOVE_BREAK_COMMAND |
 		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
+	DMA_LIST_BREAK_COMMAND =
+		LIST_BREAK_COMMAND |
+		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
 	DMA_INFO_SHAREDLIB_COMMAND =
 		INFO_SHAREDLIB_COMMAND |
 		NEED_PROGRAM_LOADED | NEED_PROGRAM_STOPPED,
@@ -202,6 +199,15 @@ typedef enum
 	DMA_RUN_TO_COMMAND =
 		RUN_TO_COMMAND | RUN_PROGRAM |
 		NEED_PROGRAM_STOPPED,
+	DMA_STEPI_IN_COMMAND =
+		STEPI_IN_COMMAND | RUN_PROGRAM |
+		NEED_PROGRAM_STOPPED,
+	DMA_STEPI_OVER_COMMAND =
+		STEPI_OVER_COMMAND | RUN_PROGRAM |
+		NEED_PROGRAM_STOPPED,
+	DMA_RUN_TO_ADDRESS_COMMAND =
+		RUN_TO_ADDRESS_COMMAND | RUN_PROGRAM |
+		NEED_PROGRAM_STOPPED,
 	DMA_EXIT_COMMAND =
 		EXIT_COMMAND | LOAD_PROGRAM |
 		NEED_PROGRAM_STOPPED,
@@ -258,7 +264,7 @@ typedef enum
 		NEED_PROGRAM_STOPPED,
 	DMA_CREATE_VARIABLE_COMMAND =
 	   CREATE_VARIABLE |
-		NEED_PROGRAM_STOPPED,
+		NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
 	DMA_EVALUATE_VARIABLE_COMMAND =
 	    EVALUATE_VARIABLE | CANCEL_IF_PROGRAM_RUNNING |
 	    NEED_PROGRAM_STOPPED,
@@ -267,7 +273,7 @@ typedef enum
 		NEED_PROGRAM_STOPPED,
 	DMA_DELETE_VARIABLE_COMMAND =
 		DELETE_VARIABLE |
-		NEED_PROGRAM_STOPPED,
+		NEED_PROGRAM_STOPPED | NEED_PROGRAM_RUNNING,
 	DMA_ASSIGN_VARIABLE_COMMAND =
 		ASSIGN_VARIABLE |
 		NEED_PROGRAM_STOPPED,
@@ -289,19 +295,21 @@ struct _DmaQueueCommand
 			gchar *file;
 			gchar *type;
 			GList *dirs;
-			gboolean terminal;
 		} load;
 		struct {
 			gchar *args;
+			gboolean terminal;
+			gboolean stop;
 		} start;
 		struct {
 			pid_t pid;
 			GList *dirs;
 		} attach;
 		struct {
+			guint *id;
 			gchar *file;
 			guint line;
-			guint address;
+			gulong address;
 			gchar *function;
 		} pos;
 		struct {
@@ -334,13 +342,15 @@ struct _DmaQueueCommand
 			guint frame;
 		} frame;
 		struct {
-			guint address;
+			gulong address;
 			guint length;
 		} mem;
 		struct {
 			gchar *name;
 			gchar *value;
 		} var;
+		gchar **env;
+		gchar *dir;
 	} data;
 	struct _DmaQueueCommand *next;
 };
@@ -353,6 +363,7 @@ dma_command_new (DmaDebuggerCommand cmd_type,...)
 {
 	DmaQueueCommand* cmd;
 	DmaDebuggerCommandType type = cmd_type & COMMAND_MASK;
+	IAnjutaDebuggerRegisterData* reg;
 	va_list args;
 	GList *list;
 	
@@ -377,7 +388,6 @@ dma_command_new (DmaDebuggerCommand cmd_type,...)
 			cmd->data.load.dirs = g_list_prepend (cmd->data.load.dirs, g_strdup (list->data));
 		}
 		cmd->data.load.dirs = g_list_reverse (cmd->data.load.dirs);
-		cmd->data.load.terminal = va_arg (args, gboolean);
 		break;
 	case ATTACH_COMMAND:
 		cmd->data.attach.pid = va_arg (args,pid_t);
@@ -394,8 +404,16 @@ dma_command_new (DmaDebuggerCommand cmd_type,...)
 		break;
 	case ABORT_COMMAND:
 		break;
+	case SET_WORKING_DIRECTORY_COMMAND:
+		cmd->data.dir = g_strdup (va_arg (args, gchar *));
+		break;
+	case SET_ENVIRONMENT_COMMAND:
+		cmd->data.env = g_strdupv (va_arg (args, gchar **));
+		break;
 	case START_COMMAND:
 		cmd->data.start.args = g_strdup (va_arg (args, gchar *));
+		cmd->data.start.terminal = va_arg (args, gboolean);
+		cmd->data.start.stop = va_arg (args, gboolean);
 	    break;
 	case RUN_COMMAND:
 		break;
@@ -409,6 +427,13 @@ dma_command_new (DmaDebuggerCommand cmd_type,...)
 		break;
 	case STEP_OUT_COMMAND:
 		break;
+	case RUN_TO_ADDRESS_COMMAND:
+		cmd->data.pos.address = va_arg (args, guint);
+		break;
+	case STEPI_IN_COMMAND:
+		break;
+	case STEPI_OVER_COMMAND:
+		break;
 	case EXIT_COMMAND:
 		break;
 	case INTERRUPT_COMMAND:
@@ -431,19 +456,22 @@ dma_command_new (DmaDebuggerCommand cmd_type,...)
 		cmd->user_data = va_arg (args, gpointer);
 		break;
 	case BREAK_LINE_COMMAND:
+		cmd->data.pos.id = va_arg (args, guint *);
 		cmd->data.pos.file = g_strdup (va_arg (args, gchar *));
 		cmd->data.pos.line = va_arg (args, guint);
 		cmd->callback = va_arg (args, IAnjutaDebuggerCallback);
 		cmd->user_data = va_arg (args, gpointer);
 		break;
 	case BREAK_FUNCTION_COMMAND:
+		cmd->data.pos.id = va_arg (args, guint *);
 		cmd->data.pos.file = g_strdup (va_arg (args, gchar *));
 		cmd->data.pos.function = g_strdup (va_arg (args, gchar *));
 		cmd->callback = va_arg (args, IAnjutaDebuggerCallback);
 		cmd->user_data = va_arg (args, gpointer);
 		break;
 	case BREAK_ADDRESS_COMMAND:
-		cmd->data.pos.address = va_arg (args, guint);
+		cmd->data.pos.id = va_arg (args, guint *);
+		cmd->data.pos.address = va_arg (args, gulong);
 		cmd->callback = va_arg (args, IAnjutaDebuggerCallback);
 		cmd->user_data = va_arg (args, gpointer);
 		break;
@@ -453,6 +481,10 @@ dma_command_new (DmaDebuggerCommand cmd_type,...)
 		cmd->callback = va_arg (args, IAnjutaDebuggerCallback);
 		cmd->user_data = va_arg (args, gpointer);
 		break;
+	case LIST_BREAK_COMMAND:
+		cmd->callback = va_arg (args, IAnjutaDebuggerCallback);
+		cmd->user_data = va_arg (args, gpointer);
+		break;
 	case INSPECT_COMMAND:
 		cmd->data.watch.name = g_strdup (va_arg (args, gchar *));
 		cmd->callback = va_arg (args, IAnjutaDebuggerCallback);
@@ -532,18 +564,19 @@ dma_command_new (DmaDebuggerCommand cmd_type,...)
 		cmd->user_data = va_arg (args, gpointer);
 		break;
 	case WRITE_REGISTER_COMMAND:
-		cmd->data.watch.id = va_arg (args, guint);
-	    cmd->data.watch.name = g_strdup (va_arg (args, gchar *));
-	    cmd->data.watch.value = g_strdup (va_arg (args, gchar *));
+		reg = va_arg (args, IAnjutaDebuggerRegisterData *);
+		cmd->data.watch.id = reg->num;
+	    cmd->data.watch.name = g_strdup (reg->name);
+	    cmd->data.watch.value = g_strdup (reg->value);
 		break;
 	case INSPECT_MEMORY_COMMAND:
-		cmd->data.mem.address = va_arg (args, guint);
+		cmd->data.mem.address = va_arg (args, gulong);
 	    cmd->data.mem.length = va_arg (args, guint);
 		cmd->callback = va_arg (args, IAnjutaDebuggerCallback);
 		cmd->user_data = va_arg (args, gpointer);
 		break;
 	case DISASSEMBLE_COMMAND:
-		cmd->data.mem.address = va_arg (args, guint);
+		cmd->data.mem.address = va_arg (args, gulong);
 	    cmd->data.mem.length = va_arg (args, guint);
 		cmd->callback = va_arg (args, IAnjutaDebuggerCallback);
 		cmd->user_data = va_arg (args, gpointer);
@@ -597,11 +630,11 @@ dma_command_new (DmaDebuggerCommand cmd_type,...)
  *---------------------------------------------------------------------------*/
 
 gboolean
-dma_queue_load (DmaDebuggerQueue *self, const gchar *file, const gchar* mime_type, const GList *search_dirs, gboolean terminal)
+dma_queue_load (DmaDebuggerQueue *self, const gchar *file, const gchar* mime_type, const GList *search_dirs)
 {
 	if (!dma_debugger_queue_start (self, mime_type)) return FALSE;
 	
-	return dma_debugger_queue_append (self, dma_command_new (DMA_LOAD_COMMAND, file, mime_type, search_dirs, terminal));
+	return dma_debugger_queue_append (self, dma_command_new (DMA_LOAD_COMMAND, file, mime_type, search_dirs));
 }
 
 gboolean
@@ -613,9 +646,21 @@ dma_queue_attach (DmaDebuggerQueue *self, pid_t pid, const GList *search_dirs)
 }
 
 gboolean
-dma_queue_start (DmaDebuggerQueue *self, const gchar *args)
+dma_queue_set_working_directory(DmaDebuggerQueue *self, const gchar *directory)
 {
-	return dma_debugger_queue_append (self, dma_command_new (DMA_START_COMMAND, args));
+	return dma_debugger_queue_append (self, dma_command_new (DMA_SET_WORKING_DIRECTORY_COMMAND, directory));
+}
+
+gboolean
+dma_queue_set_environment(DmaDebuggerQueue *self, gchar **variables)
+{
+	return dma_debugger_queue_append (self, dma_command_new (DMA_SET_ENVIRONMENT_COMMAND, variables));
+}
+
+gboolean
+dma_queue_start (DmaDebuggerQueue *self, const gchar *args, gboolean terminal, gboolean stop)
+{
+	return dma_debugger_queue_append (self, dma_command_new (DMA_START_COMMAND, args, terminal, stop));
 }
 
 gboolean
@@ -666,6 +711,24 @@ dma_queue_step_out (DmaDebuggerQueue *self)
 	return dma_debugger_queue_append (self, dma_command_new (DMA_STEP_OUT_COMMAND));
 }
 
+gboolean
+dma_queue_stepi_in (DmaDebuggerQueue *self)
+{
+	return dma_debugger_queue_append (self, dma_command_new (DMA_STEPI_IN_COMMAND));
+}
+
+gboolean
+dma_queue_stepi_over (DmaDebuggerQueue *self)
+{
+	return dma_debugger_queue_append (self, dma_command_new (DMA_STEPI_OVER_COMMAND));
+}
+
+gboolean
+dma_queue_run_to_address (DmaDebuggerQueue *self, gulong address)
+{
+	return dma_debugger_queue_append (self, dma_command_new (DMA_RUN_TO_ADDRESS_COMMAND, address));
+}
+
 gboolean
 dma_queue_exit (DmaDebuggerQueue *self)
 {
@@ -805,21 +868,21 @@ dma_queue_callback (DmaDebuggerQueue *self, IAnjutaDebuggerCallback callback , g
 }
 
 gboolean
-dma_queue_add_breakpoint_at_line (DmaDebuggerQueue *self, const gchar* file, guint line, IAnjutaDebuggerCallback callback, gpointer user_data)
+dma_queue_add_breakpoint_at_line (DmaDebuggerQueue *self, guint *id, const gchar* file, guint line, IAnjutaDebuggerCallback callback, gpointer user_data)
 {
-	return dma_debugger_queue_append (self, dma_command_new (DMA_BREAK_LINE_COMMAND, file, line, callback, user_data));
+	return dma_debugger_queue_append (self, dma_command_new (DMA_BREAK_LINE_COMMAND, id, file, line, callback, user_data));
 }
 
 gboolean
-dma_queue_add_breakpoint_at_function (DmaDebuggerQueue *self, const gchar* file, const gchar* function, IAnjutaDebuggerCallback callback, gpointer user_data)
+dma_queue_add_breakpoint_at_function (DmaDebuggerQueue *self, guint *id, const gchar* file, const gchar* function, IAnjutaDebuggerCallback callback, gpointer user_data)
 {
-	return dma_debugger_queue_append (self, dma_command_new (DMA_BREAK_FUNCTION_COMMAND, file, function, callback, user_data));
+	return dma_debugger_queue_append (self, dma_command_new (DMA_BREAK_FUNCTION_COMMAND, id, file, function, callback, user_data));
 }
 
 gboolean
-dma_queue_add_breakpoint_at_address (DmaDebuggerQueue *self, guint address, IAnjutaDebuggerCallback callback, gpointer user_data)
+dma_queue_add_breakpoint_at_address (DmaDebuggerQueue *self, guint *id, gulong address, IAnjutaDebuggerCallback callback, gpointer user_data)
 {
-	return dma_debugger_queue_append (self, dma_command_new (DMA_BREAK_ADDRESS_COMMAND, address, callback, user_data));
+	return dma_debugger_queue_append (self, dma_command_new (DMA_BREAK_ADDRESS_COMMAND, id, address, callback, user_data));
 }
 
 gboolean
@@ -846,6 +909,12 @@ dma_queue_remove_breakpoint (DmaDebuggerQueue *self, guint id, IAnjutaDebuggerCa
 	return dma_debugger_queue_append (self, dma_command_new (DMA_REMOVE_BREAK_COMMAND, id, callback, user_data));
 }
 
+gboolean
+dma_queue_list_breakpoint (DmaDebuggerQueue *self, IAnjutaDebuggerCallback callback, gpointer user_data)
+{
+	return dma_debugger_queue_append (self, dma_command_new (DMA_LIST_BREAK_COMMAND, callback, user_data));
+}
+
 gboolean
 dma_queue_list_register (DmaDebuggerQueue *self, IAnjutaDebuggerCallback callback , gpointer user_data)
 {
@@ -859,19 +928,19 @@ dma_queue_update_register (DmaDebuggerQueue *self, IAnjutaDebuggerCallback callb
 }
 
 gboolean
-dma_queue_write_register (DmaDebuggerQueue *self, IAnjutaDebuggerRegister *value)
+dma_queue_write_register (DmaDebuggerQueue *self, IAnjutaDebuggerRegisterData *value)
 {
 	return dma_debugger_queue_append (self, dma_command_new (DMA_WRITE_REGISTER_COMMAND, value));
 }
 
 gboolean
-dma_queue_inspect_memory (DmaDebuggerQueue *self, guint address, guint length, IAnjutaDebuggerCallback callback , gpointer user_data)
+dma_queue_inspect_memory (DmaDebuggerQueue *self, gulong address, guint length, IAnjutaDebuggerCallback callback , gpointer user_data)
 {
 	return dma_debugger_queue_append (self, dma_command_new (DMA_INSPECT_MEMORY_COMMAND, address, length, callback, user_data));
 }
 
 gboolean
-dma_queue_disassemble (DmaDebuggerQueue *self, guint address, guint length, IAnjutaDebuggerCallback callback , gpointer user_data)
+dma_queue_disassemble (DmaDebuggerQueue *self, gulong address, guint length, IAnjutaDebuggerCallback callback , gpointer user_data)
 {
 	return dma_debugger_queue_append (self, dma_command_new (DMA_DISASSEMBLE_COMMAND, address, length, callback, user_data));
 }
@@ -928,11 +997,15 @@ dma_command_free (DmaQueueCommand *cmd)
 	case STEP_IN_COMMAND:
 	case STEP_OVER_COMMAND:
 	case STEP_OUT_COMMAND:
+	case STEPI_IN_COMMAND:
+	case STEPI_OVER_COMMAND:
+	case RUN_TO_ADDRESS_COMMAND:
 	case EXIT_COMMAND:
 	case INTERRUPT_COMMAND:
 	case ENABLE_BREAK_COMMAND:
 	case IGNORE_BREAK_COMMAND:
 	case REMOVE_BREAK_COMMAND:
+	case LIST_BREAK_COMMAND:
 	    break;
 	case LIST_LOCAL_COMMAND:
 	case LIST_ARG_COMMAND:
@@ -970,6 +1043,12 @@ dma_command_free (DmaQueueCommand *cmd)
         g_list_foreach (cmd->data.load.dirs, (GFunc)g_free, NULL);
         g_list_free (cmd->data.load.dirs);
 		break;
+	case SET_WORKING_DIRECTORY_COMMAND:
+		if (cmd->data.dir) g_free (cmd->data.dir);
+		break;
+	case SET_ENVIRONMENT_COMMAND:
+		g_strfreev (cmd->data.env);
+		break;
 	case ATTACH_COMMAND:
         g_list_foreach (cmd->data.attach.dirs, (GFunc)g_free, NULL);
         g_list_free (cmd->data.attach.dirs);
@@ -1024,27 +1103,57 @@ dma_command_cancel (DmaQueueCommand *cmd)
 	dma_command_free (cmd);
 }
 
+/* It is possible that the queue contains several add breakpoint command
+ * for the same one. Just before sending the command to the debugger check
+ * that the breakpoint is still not set */
+
+static gboolean
+dma_command_is_breakpoint_pending (DmaQueueCommand *cmd)
+{
+	GError *err;
+	
+	if (*cmd->data.pos.id == 0) return TRUE;		/* Breakpoint is not set, can add it */
+	
+	err	= g_error_new (IANJUTA_DEBUGGER_ERROR , IANJUTA_DEBUGGER_ALREADY_DONE, "Breakpoint is already set with id %d", *cmd->data.pos.id);
+
+	if (cmd->callback != NULL)
+	{
+		cmd->callback (NULL, cmd->user_data, err);
+	}
+	g_error_free (err);
+	
+	return FALSE;
+}
+
 gboolean
 dma_command_run (DmaQueueCommand *cmd, IAnjutaDebugger *debugger,
-				 GError **err)
+				 DmaDebuggerQueue *queue, GError **err)
 {
-	IAnjutaDebuggerRegister reg;
+	IAnjutaDebuggerRegisterData reg;
 	gboolean ret = FALSE;
 	DmaDebuggerCommandType type = cmd->type & COMMAND_MASK;
-	
+	IAnjutaDebuggerCallback callback = cmd->callback == NULL ? NULL : dma_debugger_queue_command_callback;
+					 
 	switch (type)
 	{
 	case EMPTY_COMMAND:
+		ret = TRUE;
 		break;
 	case CALLBACK_COMMAND:
-		ret = ianjuta_debugger_callback (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_callback (debugger, callback, queue, err);	
 		break;
 	case LOAD_COMMAND:
-		ret = ianjuta_debugger_load (debugger, cmd->data.load.file, cmd->data.load.type, cmd->data.load.dirs, cmd->data.load.terminal, err);
+		ret = ianjuta_debugger_load (debugger, cmd->data.load.file, cmd->data.load.type, cmd->data.load.dirs, err);
 		break;
 	case ATTACH_COMMAND:
 		ret = ianjuta_debugger_attach (debugger, cmd->data.attach.pid, cmd->data.load.dirs, err);	
 		break;
+	case SET_WORKING_DIRECTORY_COMMAND:
+		ret = ianjuta_debugger_set_working_directory (debugger, cmd->data.dir, err);	
+		break;
+	case SET_ENVIRONMENT_COMMAND:
+		ret = ianjuta_debugger_set_environment (debugger, cmd->data.env, err);	
+		break;
 	case UNLOAD_COMMAND:
 	    ret = ianjuta_debugger_unload (debugger, err);
 	    break;
@@ -1055,7 +1164,7 @@ dma_command_run (DmaQueueCommand *cmd, IAnjutaDebugger *debugger,
 		ret = ianjuta_debugger_abort (debugger, err);
 		break;
 	case START_COMMAND:
-		ret = ianjuta_debugger_start (debugger, cmd->data.start.args, err);
+		ret = ianjuta_debugger_start (debugger, cmd->data.start.args, cmd->data.start.terminal, cmd->data.start.stop, err);
 	    break;
 	case RUN_COMMAND:
 		ret = ianjuta_debugger_run (debugger, err);	
@@ -1072,6 +1181,15 @@ dma_command_run (DmaQueueCommand *cmd, IAnjutaDebugger *debugger,
 	case STEP_OUT_COMMAND:
 		ret = ianjuta_debugger_step_out (debugger, err);	
 		break;
+	case RUN_TO_ADDRESS_COMMAND:
+		ret = ianjuta_debugger_instruction_run_to_address (IANJUTA_DEBUGGER_INSTRUCTION (debugger), cmd->data.pos.address, err);	
+		break;
+	case STEPI_IN_COMMAND:
+		ret = ianjuta_debugger_instruction_step_in (IANJUTA_DEBUGGER_INSTRUCTION (debugger), err);	
+		break;
+	case STEPI_OVER_COMMAND:
+		ret = ianjuta_debugger_instruction_step_over (IANJUTA_DEBUGGER_INSTRUCTION (debugger), err);	
+		break;
 	case EXIT_COMMAND:
 		ret = ianjuta_debugger_exit (debugger, err);	
 		break;
@@ -1079,127 +1197,227 @@ dma_command_run (DmaQueueCommand *cmd, IAnjutaDebugger *debugger,
 		ret = ianjuta_debugger_interrupt (debugger, err);	
 		break;
 	case ENABLE_BREAK_COMMAND:
-		ret = ianjuta_breakpoint_debugger_enable_breakpoint (IANJUTA_BREAKPOINT_DEBUGGER (debugger), cmd->data.brk.id, cmd->data.brk.enable, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_breakpoint_enable (IANJUTA_DEBUGGER_BREAKPOINT (debugger), cmd->data.brk.id, cmd->data.brk.enable, callback, queue, err);	
 		break;
 	case IGNORE_BREAK_COMMAND:
-		ret = ianjuta_breakpoint_debugger_ignore_breakpoint (IANJUTA_BREAKPOINT_DEBUGGER (debugger), cmd->data.brk.id, cmd->data.brk.ignore, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_breakpoint_ignore (IANJUTA_DEBUGGER_BREAKPOINT (debugger), cmd->data.brk.id, cmd->data.brk.ignore, callback, queue, err);	
 		break;
 	case REMOVE_BREAK_COMMAND:
-		ret = ianjuta_breakpoint_debugger_clear_breakpoint (IANJUTA_BREAKPOINT_DEBUGGER (debugger), cmd->data.brk.id, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_breakpoint_clear (IANJUTA_DEBUGGER_BREAKPOINT (debugger), cmd->data.brk.id, callback, queue, err);	
 		break;
 	case BREAK_LINE_COMMAND:
-		ret = ianjuta_breakpoint_debugger_set_breakpoint_at_line (IANJUTA_BREAKPOINT_DEBUGGER (debugger), cmd->data.pos.file, cmd->data.pos.line, cmd->callback, cmd->user_data, err);	
+		if (dma_command_is_breakpoint_pending (cmd))
+		{	
+			ret = ianjuta_debugger_breakpoint_set_at_line (IANJUTA_DEBUGGER_BREAKPOINT (debugger), cmd->data.pos.file, cmd->data.pos.line, callback, queue, err);	
+		}
+		else
+		{
+			ret = FALSE;
+		}
 		break;
 	case BREAK_FUNCTION_COMMAND:
-		ret = ianjuta_breakpoint_debugger_set_breakpoint_at_function (IANJUTA_BREAKPOINT_DEBUGGER (debugger), cmd->data.pos.file, cmd->data.pos.function, cmd->callback, cmd->user_data, err);	
+		if (dma_command_is_breakpoint_pending (cmd))
+		{	
+			ret = ianjuta_debugger_breakpoint_set_at_function (IANJUTA_DEBUGGER_BREAKPOINT (debugger), cmd->data.pos.file, cmd->data.pos.function, callback, queue, err);	
+		}
+		else
+		{
+			ret = FALSE;
+		}
 		break;
 	case BREAK_ADDRESS_COMMAND:
-		ret = ianjuta_breakpoint_debugger_set_breakpoint_at_address (IANJUTA_BREAKPOINT_DEBUGGER (debugger), cmd->data.pos.address, cmd->callback, cmd->user_data, err);	
+		if (dma_command_is_breakpoint_pending (cmd))
+		{	
+			ret = ianjuta_debugger_breakpoint_set_at_address (IANJUTA_DEBUGGER_BREAKPOINT (debugger), cmd->data.pos.address, callback, queue, err);	
+		}
+		else
+		{
+			ret = FALSE;
+		}
 		break;
 	case CONDITION_BREAK_COMMAND:
-		ret = ianjuta_breakpoint_debugger_condition_breakpoint (IANJUTA_BREAKPOINT_DEBUGGER (debugger), cmd->data.brk.id, cmd->data.brk.condition, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_breakpoint_condition (IANJUTA_DEBUGGER_BREAKPOINT (debugger), cmd->data.brk.id, cmd->data.brk.condition, callback, queue, err);	
+		break;
+	case LIST_BREAK_COMMAND:
+		ret = ianjuta_debugger_breakpoint_list (IANJUTA_DEBUGGER_BREAKPOINT (debugger), callback, queue, err);	
 		break;
 	case INSPECT_COMMAND:
-		ret = ianjuta_debugger_inspect (debugger, cmd->data.watch.name, cmd->callback, cmd->user_data, err);
+		ret = ianjuta_debugger_inspect (debugger, cmd->data.watch.name, callback, queue, err);
 	    break;
 	case EVALUATE_COMMAND:
-		ret = ianjuta_debugger_evaluate (debugger, cmd->data.watch.name, cmd->data.watch.value, cmd->callback, cmd->user_data, err);
+		ret = ianjuta_debugger_evaluate (debugger, cmd->data.watch.name, cmd->data.watch.value, callback, queue, err);
 	    break;
 	case LIST_LOCAL_COMMAND:
-		ret = ianjuta_debugger_list_local (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_list_local (debugger, callback, queue, err);	
 		break;
 	case LIST_ARG_COMMAND:
-		ret = ianjuta_debugger_list_argument (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_list_argument (debugger, callback, queue, err);	
 		break;
 	case LIST_THREAD_COMMAND:
-		ret = ianjuta_debugger_list_thread (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_list_thread (debugger, callback, queue, err);	
 		break;
 	case SET_THREAD_COMMAND:
 		ret = ianjuta_debugger_set_thread (debugger, cmd->data.frame.frame, err);	
 		break;
 	case INFO_THREAD_COMMAND:
-		ret = ianjuta_debugger_info_thread (debugger, cmd->data.info.id, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_info_thread (debugger, cmd->data.info.id, callback, queue, err);	
 		break;
 	case INFO_SIGNAL_COMMAND:
-		ret = ianjuta_debugger_info_signal (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_info_signal (debugger, callback, queue, err);	
 		break;
 	case INFO_SHAREDLIB_COMMAND:
-		ret = ianjuta_debugger_info_sharedlib (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_info_sharedlib (debugger, callback, queue, err);	
 		break;
 	case INFO_FRAME_COMMAND:
-		ret = ianjuta_debugger_info_frame (debugger, 0, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_info_frame (debugger, 0, callback, queue, err);	
 		break;
 	case INFO_ARGS_COMMAND:
-		ret = ianjuta_debugger_info_args (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_info_args (debugger, callback, queue, err);	
 		break;
 	case INFO_TARGET_COMMAND:
-		ret = ianjuta_debugger_info_target (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_info_target (debugger, callback, queue, err);	
 		break;
 	case INFO_PROGRAM_COMMAND:
-		ret = ianjuta_debugger_info_program (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_info_program (debugger, callback, queue, err);	
 		break;
 	case INFO_UDOT_COMMAND:
-		ret = ianjuta_debugger_info_udot (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_info_udot (debugger, callback, queue, err);	
 		break;
 	case INFO_VARIABLES_COMMAND:
-		ret = ianjuta_debugger_info_variables (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_info_variables (debugger, callback, queue, err);	
 		break;
 	case SET_FRAME_COMMAND:
 		ret = ianjuta_debugger_set_frame (debugger, cmd->data.frame.frame, err);	
 		break;
 	case LIST_FRAME_COMMAND:
-		ret = ianjuta_debugger_list_frame (debugger, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_list_frame (debugger, callback, queue, err);	
 		break;
 	case LIST_REGISTER_COMMAND:
-		ret = ianjuta_cpu_debugger_list_register (IANJUTA_CPU_DEBUGGER (debugger), cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_register_list (IANJUTA_DEBUGGER_REGISTER (debugger), callback, queue, err);	
 		break;
 	case UPDATE_REGISTER_COMMAND:
-		ret = ianjuta_cpu_debugger_update_register (IANJUTA_CPU_DEBUGGER (debugger), cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_register_update (IANJUTA_DEBUGGER_REGISTER (debugger), callback, queue, err);	
 		break;
 	case WRITE_REGISTER_COMMAND:
 		reg.num = cmd->data.watch.id;
 	    reg.name = cmd->data.watch.name;
 	    reg.value = cmd->data.watch.value;
-		ret = ianjuta_cpu_debugger_write_register (IANJUTA_CPU_DEBUGGER (debugger), &reg, err);	
+		ret = ianjuta_debugger_register_write (IANJUTA_DEBUGGER_REGISTER (debugger), &reg, err);	
 		break;
 	case INSPECT_MEMORY_COMMAND:
-		ret = ianjuta_cpu_debugger_inspect_memory (IANJUTA_CPU_DEBUGGER (debugger), cmd->data.mem.address, cmd->data.mem.length, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_memory_inspect (IANJUTA_DEBUGGER_MEMORY (debugger), cmd->data.mem.address, cmd->data.mem.length, callback, queue, err);	
 		break;
 	case DISASSEMBLE_COMMAND:
-		ret = ianjuta_cpu_debugger_disassemble (IANJUTA_CPU_DEBUGGER (debugger), cmd->data.mem.address, cmd->data.mem.length, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_instruction_disassemble (IANJUTA_DEBUGGER_INSTRUCTION (debugger), cmd->data.mem.address, cmd->data.mem.length, callback, queue, err);	
 		break;
 	case USER_COMMAND:
 		ret = ianjuta_debugger_send_command (debugger, cmd->data.user.cmd, err);	
 		break;
 	case PRINT_COMMAND:
-		ret = ianjuta_debugger_print (debugger, cmd->data.print.var, cmd->callback, cmd->user_data, err);	
+		ret = ianjuta_debugger_print (debugger, cmd->data.print.var, callback, queue, err);	
 		break;
 	case HANDLE_SIGNAL_COMMAND:
 		ret = ianjuta_debugger_handle_signal (debugger, cmd->data.signal.name, cmd->data.signal.stop, cmd->data.signal.print, cmd->data.signal.ignore, err);	
 		break;
 	case DELETE_VARIABLE:
-		ret = ianjuta_variable_debugger_delete_var (IANJUTA_VARIABLE_DEBUGGER (debugger), cmd->data.var.name, NULL);
+		ret = ianjuta_debugger_variable_destroy (IANJUTA_DEBUGGER_VARIABLE (debugger), cmd->data.var.name, NULL);
 		break;
 	case ASSIGN_VARIABLE:
-		ret = ianjuta_variable_debugger_assign (IANJUTA_VARIABLE_DEBUGGER (debugger), cmd->data.var.name, cmd->data.var.value, err);
+		ret = ianjuta_debugger_variable_assign (IANJUTA_DEBUGGER_VARIABLE (debugger), cmd->data.var.name, cmd->data.var.value, err);
 		break;
 	case EVALUATE_VARIABLE:
-		ret = ianjuta_variable_debugger_evaluate (IANJUTA_VARIABLE_DEBUGGER (debugger), cmd->data.var.name, cmd->callback, cmd->user_data, err);
+		ret = ianjuta_debugger_variable_evaluate (IANJUTA_DEBUGGER_VARIABLE (debugger), cmd->data.var.name, callback, queue, err);
 		break;
 	case LIST_VARIABLE_CHILDREN:
-		ret = ianjuta_variable_debugger_list_children (IANJUTA_VARIABLE_DEBUGGER (debugger), cmd->data.var.name, cmd->callback, cmd->user_data, err);
+		ret = ianjuta_debugger_variable_list_children (IANJUTA_DEBUGGER_VARIABLE (debugger), cmd->data.var.name, callback, queue, err);
 		break;
 	case CREATE_VARIABLE:
-		ret = ianjuta_variable_debugger_create (IANJUTA_VARIABLE_DEBUGGER (debugger), cmd->data.var.name, cmd->callback, cmd->user_data, err);
+		ret = ianjuta_debugger_variable_create (IANJUTA_DEBUGGER_VARIABLE (debugger), cmd->data.var.name, callback, queue, err);
 		break;
 	case UPDATE_VARIABLE:
-		ret = ianjuta_variable_debugger_update (IANJUTA_VARIABLE_DEBUGGER (debugger), cmd->callback, cmd->user_data, err);
+		ret = ianjuta_debugger_variable_update (IANJUTA_DEBUGGER_VARIABLE (debugger), callback, queue, err);
 		break;
 	}
 	
 	return ret;
 }
 
+void
+dma_command_callback (DmaQueueCommand *cmd, const gpointer data, GError *err)
+{
+	DmaDebuggerCommandType type = cmd->type & COMMAND_MASK;
+	
+	switch (type)
+	{
+	case EMPTY_COMMAND:
+	case LOAD_COMMAND:
+	case ATTACH_COMMAND:
+	case UNLOAD_COMMAND:
+	case QUIT_COMMAND:
+	case ABORT_COMMAND:
+	case START_COMMAND:
+	case SET_WORKING_DIRECTORY_COMMAND:
+	case SET_ENVIRONMENT_COMMAND:	
+	case RUN_COMMAND:
+	case RUN_TO_COMMAND:
+	case STEP_IN_COMMAND:
+	case STEP_OVER_COMMAND:
+	case STEP_OUT_COMMAND:
+	case RUN_TO_ADDRESS_COMMAND:
+	case STEPI_IN_COMMAND:
+	case STEPI_OVER_COMMAND:
+	case EXIT_COMMAND:
+	case INTERRUPT_COMMAND:
+	case SET_THREAD_COMMAND:
+	case SET_FRAME_COMMAND:
+	case WRITE_REGISTER_COMMAND:
+	case USER_COMMAND:
+	case HANDLE_SIGNAL_COMMAND:
+	case DELETE_VARIABLE:
+	case ASSIGN_VARIABLE:
+		g_return_if_reached ();	
+		break;
+	case CALLBACK_COMMAND:
+	case ENABLE_BREAK_COMMAND:
+	case IGNORE_BREAK_COMMAND:
+	case REMOVE_BREAK_COMMAND:
+	case BREAK_LINE_COMMAND:
+	case BREAK_FUNCTION_COMMAND:
+	case BREAK_ADDRESS_COMMAND:
+	case CONDITION_BREAK_COMMAND:
+	case LIST_BREAK_COMMAND:
+	case INSPECT_COMMAND:
+	case EVALUATE_COMMAND:
+	case LIST_LOCAL_COMMAND:
+	case LIST_ARG_COMMAND:
+	case LIST_THREAD_COMMAND:
+	case INFO_THREAD_COMMAND:
+	case INFO_SIGNAL_COMMAND:
+	case INFO_SHAREDLIB_COMMAND:
+	case INFO_FRAME_COMMAND:
+	case INFO_ARGS_COMMAND:
+	case INFO_TARGET_COMMAND:
+	case INFO_PROGRAM_COMMAND:
+	case INFO_UDOT_COMMAND:
+	case INFO_VARIABLES_COMMAND:
+	case LIST_FRAME_COMMAND:
+	case LIST_REGISTER_COMMAND:
+	case UPDATE_REGISTER_COMMAND:
+	case INSPECT_MEMORY_COMMAND:
+	case DISASSEMBLE_COMMAND:
+	case PRINT_COMMAND:
+	case EVALUATE_VARIABLE:
+	case LIST_VARIABLE_CHILDREN:
+	case CREATE_VARIABLE:
+	case UPDATE_VARIABLE:
+		if (cmd->callback != NULL)
+		{
+			cmd->callback (data, cmd->user_data, err);	
+		}
+		break;
+	}
+}
+
 gboolean
 dma_command_is_valid_in_state (DmaQueueCommand *cmd, IAnjutaDebuggerState state)
 {
