@@ -59,19 +59,12 @@ static gpointer parent_class;
 #define ANJUTA_STOCK_HELP_PREV "anjuta-help-prev"
 #define ANJUTA_STOCK_DEVHELP "anjuta-devhelp"
 
-#define REGISTER_ICON(icon, stock_id) \
-	pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"icon, NULL); \
-	icon_set = gtk_icon_set_new_from_pixbuf (pixbuf); \
-	gtk_icon_factory_add (icon_factory, stock_id, icon_set); \
-	g_object_unref (pixbuf);
-
 static void
 register_stock_icons (AnjutaPlugin *plugin)
 {
 	AnjutaUI *ui;
 	GtkIconFactory *icon_factory;
 	GtkIconSet *icon_set;
-	GdkPixbuf *pixbuf;
 	static gboolean registered = FALSE;
 
 	if (registered)
@@ -266,7 +259,11 @@ static GtkActionEntry actions[] = {
 	},
 	{
 		"ActionHelpContext",
+#ifndef DISABLE_EMBEDDED_DEVHELP
 		ANJUTA_STOCK_DEVHELP,
+#else
+		NULL,
+#endif /* DISABLE_EMBEDDED_DEVHELP */
 		N_("_Context Help"),
 		"<shift>F1",
 		N_("Search help for the current word in the editor"),
@@ -320,9 +317,9 @@ devhelp_activate (AnjutaPlugin *plugin)
 
 	AnjutaUI *ui;
 	AnjutaDevhelp *devhelp;
-	static gboolean init = FALSE;
 
 #ifndef DISABLE_EMBEDDED_DEVHELP
+	static gboolean init = FALSE;
 	GNode *books;
 	GList *keywords;
 	GtkWidget* books_sw;
@@ -385,6 +382,7 @@ devhelp_activate (AnjutaPlugin *plugin)
 	anjuta_shell_add_widget (plugin->shell, devhelp->control_notebook,
 								 "AnjutaDevhelpIndex", _("Help"), ANJUTA_STOCK_DEVHELP,
 								 ANJUTA_SHELL_PLACEMENT_LEFT, NULL);
+	/* This is the window that show the html help text */
 	anjuta_shell_add_widget (plugin->shell, devhelp->htmlview,
 								 "AnjutaDevhelpDisplay", _("Help display"),
 								 ANJUTA_STOCK_DEVHELP,
@@ -459,7 +457,7 @@ devhelp_finalize (GObject *obj)
 	/* Finalization codes here */
 	AnjutaDevhelp *plugin = ANJUTA_PLUGIN_DEVHELP (obj);
 	
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -471,7 +469,7 @@ devhelp_dispose (GObject *obj)
 	// g_object_unref(G_OBJECT(devhelp->base));
 
 	/* Disposition codes */
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 #endif
 
