@@ -38,8 +38,6 @@
 #include <libanjuta/interfaces/ianjuta-editor-goto.h>
 #include <libanjuta/interfaces/ianjuta-editor-language.h>
 
-#include <libegg/menu/egg-entry-action.h>
-
 #include <sys/wait.h>
 #include <sys/stat.h>
 
@@ -251,17 +249,6 @@ on_close_all_file_activate (GtkAction *action, gpointer user_data)
 	}
 }
 
-void
-on_new_file_activate (GtkAction *action, gpointer user_data)
-{
-	AnjutaDocman *docman;
-	DocmanPlugin *plugin;
-	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
-	docman = ANJUTA_DOCMAN (plugin->docman);
-
-	anjuta_docman_add_editor (docman, NULL, NULL);
-}
-
 void
 on_reload_file_activate (GtkAction *action, gpointer user_data)
 {
@@ -1067,3 +1054,25 @@ on_editor_remove_view_activate (GtkAction *action, gpointer user_data)
 	if (doc)
 		ianjuta_editor_view_remove_current (IANJUTA_EDITOR_VIEW (doc), NULL);
 }
+
+void on_next_document (GtkAction *action, gpointer user_data)
+{
+	AnjutaDocman *docman;
+	DocmanPlugin *plugin;
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	docman = ANJUTA_DOCMAN (plugin->docman);
+	
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (docman),
+								   gtk_notebook_get_current_page (GTK_NOTEBOOK(docman)) + 1);
+}
+
+void on_previous_document (GtkAction *action, gpointer user_data)
+{
+	AnjutaDocman *docman;
+	DocmanPlugin *plugin;
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	docman = ANJUTA_DOCMAN (plugin->docman);
+	
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (docman),
+								   gtk_notebook_get_current_page (GTK_NOTEBOOK(docman)) - 1);
+}
