@@ -15,31 +15,22 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <config.h>
 
-#if (GLADEUI_VERSION <= 302)
-# include <glade.h>
-#else
-# if (GLADEUI_VERSION <= 314)
-#   include <glade.h>
-#   include <glade-design-view.h>
-# else /* Since 3.1.5 */
-#   include <gladeui/glade.h>
-#   include <gladeui/glade-design-view.h>
-# endif
-#endif
-
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
+#include <libanjuta/interfaces/ianjuta-file-savable.h>
 #include <libanjuta/interfaces/ianjuta-wizard.h>
 #include <libanjuta/interfaces/ianjuta-help.h>
+#include <libanjuta/interfaces/ianjuta-document-manager.h>
 
 #include "plugin.h"
+#include "anjuta-design-document.h"
 
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-glade.ui"
 
@@ -54,11 +45,10 @@ struct _GladePluginPriv
 	GtkActionGroup *action_group;
 	GladeApp  *gpw;
 	GtkWidget *inspector;
-#if (GLADEUI_VERSION > 302)
-	GtkWidget *design_notebook;
-#endif
 	GtkWidget *view_box;
 	GtkWidget *projects_combo;
+	gint editor_watch_id;
+	gboolean destroying;
 };
 
 enum {
@@ -67,329 +57,72 @@ enum {
 	N_COLUMNS
 };
 
-static void
-glade_update_ui (GladeApp *app, GladePlugin *plugin)
-{
-#if (GLADEUI_VERSION >= 330)
-	GladeCommand *undo_item;
-	GladeCommand *redo_item;
-#else
-	GList *prev_redo_item;
-	GList *undo_item;
-	GList *redo_item;
-#endif
-	const gchar *undo_description = NULL;
-	const gchar *redo_description = NULL;
-	GladeProject *project;
-	GtkAction *action;
-	gchar buffer[512];
-	AnjutaUI *ui;
-	GtkTreeModel *model;
-	GtkTreeIter iter;;
-
-	project = glade_app_get_project ();
-	if (!project)
-	{
-		undo_item = NULL;
-		redo_item = NULL;
-	}
-	else
-	{
-#if (GLADEUI_VERSION >= 330)
-		undo_item = glade_project_next_undo_item(project);
-		redo_item = glade_project_next_redo_item(project);
-		if (undo_item)
-			undo_description = undo_item->description;
-		if (redo_item)
-			redo_description = redo_item->description;
-#else
-		undo_item = prev_redo_item = project->prev_redo_item;
-		redo_item = (prev_redo_item == NULL) ? project->undo_stack : prev_redo_item->next;
-		if (undo_item && undo_item->data)
-			undo_description = GLADE_COMMAND (undo_item->data)->description;
-		if (redo_item && redo_item->data)
-			redo_description = GLADE_COMMAND (redo_item->data)->description;
-#endif
-	}
-	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (plugin)->shell, NULL);
-	action = anjuta_ui_get_action (ui, "ActionGroupGlade", "ActionGladeUndo");
-	if (undo_description)
-	{
-		snprintf (buffer, 512, _("_Undo: %s"), undo_description);
-		g_object_set (G_OBJECT (action), "sensitive", TRUE, "label", buffer, NULL);
-	}
-	else
-	{
-		g_object_set (G_OBJECT (action), "sensitive", FALSE, "label", _("_Undo"), NULL);
-	}
-	action = anjuta_ui_get_action (ui, "ActionGroupGlade", "ActionGladeRedo");
-	if (redo_description)
-	{
-		snprintf (buffer, 512, _("_Redo: %s"), redo_description);
-		g_object_set (G_OBJECT (action), "sensitive", TRUE, "label", buffer, NULL);
-	}
-	else
-	{
-		g_object_set (G_OBJECT (action), "sensitive", FALSE, "label", _("_Redo"), NULL);
-	}
+static void 
+update_current_project (GtkComboBox *projects_combo,
+                        GladeProject* project)
+{	
+	GtkTreeIter iter;
+	GladeProject *cur_project;	
+	GtkTreeModel* model = gtk_combo_box_get_model (projects_combo);
 	
-	/* Update current project */
-	model = gtk_combo_box_get_model (GTK_COMBO_BOX (plugin->priv->projects_combo));
 	if (gtk_tree_model_get_iter_first (model, &iter))
-	{
 		do
 		{
-			GladeProject *project;
-			gtk_tree_model_get (model, &iter, PROJECT_COL, &project, -1);
-			if (project == glade_app_get_project ())
+			gtk_tree_model_get (model, &iter, PROJECT_COL, &cur_project, -1);
+			if (project == cur_project)
 			{
-				gtk_combo_box_set_active_iter (GTK_COMBO_BOX (plugin->priv->projects_combo), &iter);
+				gtk_combo_box_set_active_iter (projects_combo, &iter);
 				break;
 			}
 		}
 		while (gtk_tree_model_iter_next (model, &iter));
-	}
-}
-
-/* Action callbacks */
-static void
-on_copy_activated (GtkAction *action, GladePlugin *plugin)
-{
-	glade_app_command_copy ();
 }
 
 static void
-on_cut_activated (GtkAction *action, GladePlugin *plugin)
+value_added_current_editor (AnjutaPlugin *plugin, const char *name,
+							const GValue *value, gpointer data)
 {
-	glade_app_command_cut ();
-}
-
-static void
-on_paste_activated (GtkAction *action, GladePlugin *plugin)
-{
-	glade_app_command_paste (NULL);
-}
-
-static void
-on_delete_activated (GtkAction *action, GladePlugin *plugin)
-{
-	glade_app_command_delete ();
-}
-
-static void
-on_undo_activated (GtkAction *action, GladePlugin *plugin)
-{
-	glade_app_command_undo ();
-}
-
-static void
-on_redo_activated (GtkAction *action, GladePlugin *plugin)
-{
-	glade_app_command_redo ();
-}
-
-static void
-on_show_clipbard_activated (GtkAction *action, GladePlugin *plugin)
-{
-	GtkWidget *clipboard_view;
-	
-	clipboard_view = glade_app_get_clipboard_view ();
-	gtk_widget_show_all (clipboard_view);
-	gtk_window_present (GTK_WINDOW (clipboard_view));
-}
-
-static void
-glade_save (GladePlugin *plugin, GladeProject *project, const gchar *path)
-{
-	AnjutaStatus *status;
-	
-	status = anjuta_shell_get_status (ANJUTA_PLUGIN(plugin)->shell, NULL);
-	if (!glade_project_save (project, path, NULL))
+	GladePlugin* glade_plugin = ANJUTA_PLUGIN_GLADE(plugin);
+	GObject *editor;	
+	editor = g_value_get_object (value);
+	if (ANJUTA_IS_DESIGN_DOCUMENT(editor))
 	{
-		anjuta_util_dialog_warning (GTK_WINDOW (ANJUTA_PLUGIN(plugin)->shell),
-									_("Invalid glade file name"));
-		return;
+		AnjutaDesignDocument* view = ANJUTA_DESIGN_DOCUMENT(editor);
+		GladeProject* project = glade_design_view_get_project(GLADE_DESIGN_VIEW(view));
+		if (!view->is_project_added)
+		{
+			glade_app_add_project (project);
+			view->is_project_added = TRUE;
+		}
+		glade_app_set_project (project);
 	}
-	// FIXME: gpw_refresh_project_entry (gpw, project);
-#if (GLADEUI_VERSION >= 330)
-	anjuta_status_set (status, _("Glade project '%s' saved"),
-					   glade_project_get_name(project));
-#else
-	anjuta_status_set (status, _("Glade project '%s' saved"), project->name);
-#endif
 }
 
 static void
-on_save_activated (GtkAction *action, GladePlugin *plugin)
+value_removed_current_editor (AnjutaPlugin *plugin,
+							  const char *name, gpointer data)
 {
-	GladeProject *project;
-	GtkWidget *filechooser;
-	gchar *path = NULL;
-
-	project = glade_app_get_project ();
-
-#if (GLADEUI_VERSION >= 330)
-	if (glade_project_get_path(project) != NULL) {
-#else
-	if (project->path != NULL) {
-#endif
-		AnjutaStatus *status;
-		
-		status = anjuta_shell_get_status (ANJUTA_PLUGIN(plugin)->shell, NULL);
-		
-#if (GLADEUI_VERSION >= 330)
-		if (glade_project_save (project, glade_project_get_path(project),
-								NULL)) {
-			anjuta_status_set (status, _("Glade project '%s' saved"),
-							   glade_project_get_name(project));
-#else	
-		if (glade_project_save (project, project->path, NULL)) {
-			anjuta_status_set (status, _("Glade project '%s' saved"),
-							   project->name);
-#endif
-		} else {
-			anjuta_util_dialog_warning (GTK_WINDOW (ANJUTA_PLUGIN(plugin)->shell),
-										_("Invalid glade file name"));
-		}
-		return;
-	}
-
-	/* If instead we dont have a path yet, fire up a file selector */
-	filechooser = glade_util_file_dialog_new (_("Save glade file..."),
-									GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
-											  GLADE_FILE_DIALOG_ACTION_SAVE);
 	
-	if (gtk_dialog_run (GTK_DIALOG(filechooser)) == GTK_RESPONSE_OK)
-		path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooser));
-
-	gtk_widget_destroy (filechooser);
-
-	if (!path)
-		return;
-
-	glade_save (plugin, project, path);
-	g_free (path);
 }
 
 static void
-on_save_as_activated (GtkAction *action, GladePlugin *plugin)
+glade_update_ui (GladeApp *app, GladePlugin *plugin)
 {
-	GladeProject *project;
-	GtkWidget *filechooser;
-	gchar *path = NULL;
-
-	project = glade_app_get_project ();
-
-	filechooser = glade_util_file_dialog_new (_("Save glade file as..."),
-									GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
-											  GLADE_FILE_DIALOG_ACTION_SAVE);
-
-	if (gtk_dialog_run (GTK_DIALOG(filechooser)) == GTK_RESPONSE_OK)
-		path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooser));
-
-	gtk_widget_destroy (filechooser);
-
-	if (!path)
-		return;
-
-	glade_save (plugin, project, path);
-	g_free (path);
-}
-
-static gboolean
-glade_confirm_close_project (GladePlugin *plugin, GladeProject *project)
-{
-	GladeApp *gpw;
-	GtkWidget *dialog;
-	gboolean close = FALSE;
-	gchar *msg, *string; 
-	gint ret;
-
-	g_return_val_if_fail (GLADE_IS_PROJECT (project), FALSE);
-
-	gpw = plugin->priv->gpw;
-	string = g_strdup_printf(
-					"<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s", 				
-					_("Save changes to glade project \"%s\" before closing?"), 
-					_("Your changes will be lost if you don't save them."));
-#if (GLADEUI_VERSION >= 330)
-	msg = g_strdup_printf (string, glade_project_get_name(project));
-#else
-	msg = g_strdup_printf (string, project->name);
-#endif
-	g_free(string);
-	dialog = gtk_message_dialog_new (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_QUESTION,
-					 GTK_BUTTONS_NONE,
-					 msg);
-	g_free(msg);
-
-	gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
-
-	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-				_("_Close without Saving"), GTK_RESPONSE_NO,
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				GTK_STOCK_SAVE, GTK_RESPONSE_YES, NULL);
-
-	gtk_dialog_set_default_response	(GTK_DIALOG (dialog), GTK_RESPONSE_YES);
-
-	ret = gtk_dialog_run (GTK_DIALOG (dialog));
-	switch (ret) {
-	case GTK_RESPONSE_YES:
-		/* if YES we save the project: note we cannot use gpw_save_cb
-		 * since it saves the current project, while the modified 
-                 * project we are saving may be not the current one.
-		 */
-#if (GLADEUI_VERSION >= 330)
-		if (glade_project_get_path(project) != NULL)
- 		{
-			close = glade_project_save (project,
-										glade_project_get_path(project),
-										NULL);
- 		}
-#else
-		if (project->path != NULL)
-		{
-			close = glade_project_save (project, project->path, NULL);
-		}
-#endif
-		else
-		{
-			GtkWidget *filechooser;
-			gchar *path = NULL;
-
-			filechooser = glade_util_file_dialog_new (_("Save glade project..."),
-									GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
-								   GLADE_FILE_DIALOG_ACTION_SAVE);
-
-			if (gtk_dialog_run (GTK_DIALOG(filechooser)) == GTK_RESPONSE_OK)
-				path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooser));
-
-			gtk_widget_destroy (filechooser);
-
-			if (!path)
-				break;
-
-			glade_save (plugin, project, path);
-			g_free (path);
-			close = FALSE;
-		}
-		break;
-	case GTK_RESPONSE_NO:
-		close = TRUE;
-		break;
-	case GTK_RESPONSE_CANCEL:
-	case GTK_RESPONSE_DELETE_EVENT:
-		close = FALSE;
-		break;
-	default:
-		g_assert_not_reached ();
-		close = FALSE;
+	IAnjutaDocument* doc;
+	IAnjutaDocumentManager* docman = 
+		anjuta_shell_get_interface(ANJUTA_PLUGIN(plugin)->shell,
+								   IAnjutaDocumentManager, NULL);
+	
+	update_current_project (GTK_COMBO_BOX (plugin->priv->projects_combo), glade_app_get_project ());
+	/* Emit IAnjutaDocument signal */	
+	doc = ianjuta_document_manager_get_current_document(docman, NULL);
+	if (doc && ANJUTA_IS_DESIGN_DOCUMENT(doc))
+	{
+		gboolean dirty = ianjuta_file_savable_is_dirty(IANJUTA_FILE_SAVABLE(doc), NULL);
+		g_signal_emit_by_name (G_OBJECT(doc), "update_ui");
+		g_signal_emit_by_name (G_OBJECT(doc), "save_point", !dirty);
 	}
-
-	gtk_widget_destroy (dialog);
-	return close;
+	
 }
 
 static void
@@ -428,47 +161,30 @@ on_api_help (GladeEditor* editor,
 static void
 glade_do_close (GladePlugin *plugin, GladeProject *project)
 {
-#if (GLADEUI_VERSION > 302)
-	GtkWidget *design_view;
-
-	design_view = g_object_get_data (G_OBJECT (project), "design_view");
-	gtk_notebook_remove_page (GTK_NOTEBOOK (plugin->priv->design_notebook),
-							  gtk_notebook_page_num (GTK_NOTEBOOK
-													 (plugin->priv->design_notebook),
-													 design_view));
-	gtk_widget_destroy (design_view);
-#endif
 	glade_app_remove_project (project);
 }
 
 static void
-on_close_activated (GtkAction *action, GladePlugin *plugin)
+on_document_destroy (GtkWidget* document, GladePlugin *plugin)
 {
 	GladeProject *project;
-	gboolean close;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	
-	project = glade_app_get_project ();
 
-	if (!project)
+	DEBUG_PRINT(__FUNCTION__);
+	
+	if (plugin->priv->destroying)
 	{
-		/* if (gtk_tree_model_iter_n_children (model, NULL) <= 0) */
-		anjuta_plugin_deactivate (ANJUTA_PLUGIN (plugin));
 		return;
 	}
 	
-#if (GLADEUI_VERSION >= 330)
-	if (glade_project_get_has_unsaved_changes (project))
-#else
-	if (project->changed)
-#endif
+	project = glade_design_view_get_project(GLADE_DESIGN_VIEW(document));
+
+	if (!project)
 	{
-		close = glade_confirm_close_project (plugin, project);
-			if (!close)
-				return;
+		return;
 	}
-	
+		
 	/* Remove project from our list */
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (plugin->priv->projects_combo));
 	if (gtk_tree_model_get_iter_first (model, &iter))
@@ -487,148 +203,48 @@ on_close_activated (GtkAction *action, GladePlugin *plugin)
 		while (gtk_tree_model_iter_next (model, &iter));
 	}
 	glade_do_close (plugin, project);
+
 	if (gtk_tree_model_iter_n_children (model, NULL) <= 0)
 		anjuta_plugin_deactivate (ANJUTA_PLUGIN (plugin));
 }
 
+static void
+on_shell_destroy (AnjutaShell* shell, GladePlugin *glade_plugin)
+{
+	glade_plugin->priv->destroying = TRUE;
+}
+
 static void
 on_glade_project_changed (GtkComboBox *combo, GladePlugin *plugin)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
+	IAnjutaDocumentManager* docman = 
+		anjuta_shell_get_interface(ANJUTA_PLUGIN(plugin)->shell,
+								   IAnjutaDocumentManager, NULL);
 	
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (plugin->priv->projects_combo));
 	if (gtk_combo_box_get_active_iter (combo, &iter))
 	{
 		GladeProject *project;
 		
-#if (GLADEUI_VERSION > 302)
 		GtkWidget *design_view;
-		gint design_pagenum;
-#endif
 		gtk_tree_model_get (model, &iter, PROJECT_COL, &project, -1);
 		glade_app_set_project (project);
 		
-#if (GLADEUI_VERSION > 302)
 		design_view = g_object_get_data (G_OBJECT (project), "design_view");
-		design_pagenum = gtk_notebook_page_num (GTK_NOTEBOOK (plugin->priv->design_notebook),
-												design_view);
-		gtk_notebook_set_current_page (GTK_NOTEBOOK (plugin->priv->design_notebook),
-									   design_pagenum);
+		ianjuta_document_manager_set_current_document(docman, IANJUTA_DOCUMENT(design_view), NULL);
+		
 #  if (GLADEUI_VERSION >= 330)
         glade_inspector_set_project (GLADE_INSPECTOR (plugin->priv->inspector), project);
 #  endif								   
 									   
-#endif
 	}
 }
 
-/* Action definitions */
-static GtkActionEntry actions[] = {
-	/* Go menu */
-	{
-		"ActionMenuGlade",
-		NULL,
-		N_("_Glade"),
-		NULL,
-		NULL,
-		NULL
-	},
-	{
-		"ActionGladeUndo",
-		GTK_STOCK_UNDO,
-		N_("_Undo"),
-		NULL,
-		N_("Undo last action"),
-		G_CALLBACK (on_undo_activated)
-	},
-	{
-		"ActionGladeRedo",
-		GTK_STOCK_REDO,
-		N_("_Redo"),
-		NULL,
-		N_("Redo last undone action"),
-		G_CALLBACK (on_redo_activated)
-	},
-	{
-		"ActionGladeCut",
-		GTK_STOCK_CUT,
-		N_("Cu_t"),
-		NULL,
-		N_("Cut selection"),
-		G_CALLBACK (on_cut_activated)
-	},
-	{
-		"ActionGladeCopy",
-		GTK_STOCK_COPY,
-		N_("_Copy"),
-		"",
-		N_("Copy selection"),
-		G_CALLBACK (on_copy_activated)
-	},
-	{
-		"ActionGladePaste",
-		GTK_STOCK_PASTE,
-		N_("_Paste"),
-		"",
-		N_("Paste selection"),
-		G_CALLBACK (on_paste_activated)
-	},
-	{
-		"ActionGladeDelete",
-		GTK_STOCK_DELETE,
-		N_("_Delete"),
-		"",
-		N_("Delete selection"),
-		G_CALLBACK (on_delete_activated)
-	},
-	{
-		"ActionGladeShowClipboard",
-		NULL,
-		N_("_Show Clipboard"),
-		NULL,
-		N_("Show clipboard"),
-		G_CALLBACK (on_show_clipbard_activated)
-	},
-	{
-		"ActionGladeSave",
-		GTK_STOCK_SAVE,
-		N_("_Save"),
-		"",
-		N_("Save glade project"),
-		G_CALLBACK (on_save_activated)
-	},
-	{
-		"ActionGladeSaveAs",
-		GTK_STOCK_SAVE_AS,
-		N_("Save _As..."),
-		"",
-		N_("Save as glade project"),
-		G_CALLBACK (on_save_as_activated)
-	},
-	{
-		"ActionGladeClose",
-		GTK_STOCK_CLOSE,
-		N_("Clos_e"),
-		"",
-		N_("Close current glade project"),
-		G_CALLBACK (on_close_activated)
-	}
-};
-
-#define REGISTER_ICON(icon, stock_id) \
-	pixbuf = gdk_pixbuf_new_from_file (icon, NULL); \
-	icon_set = gtk_icon_set_new_from_pixbuf (pixbuf); \
-	gtk_icon_factory_add (icon_factory, stock_id, icon_set); \
-	g_object_unref (pixbuf);
-
 static void
 register_stock_icons (AnjutaPlugin *plugin)
 {
-	AnjutaUI *ui;
-	GtkIconFactory *icon_factory;
-	GtkIconSet *icon_set;
-	GdkPixbuf *pixbuf;
 	static gboolean registered = FALSE;
 
 	if (registered)
@@ -636,108 +252,124 @@ register_stock_icons (AnjutaPlugin *plugin)
 	registered = TRUE;
 
 	/* Register stock icons */
-	ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	icon_factory = anjuta_ui_get_icon_factory (ui);
-	REGISTER_ICON (PACKAGE_PIXMAPS_DIR"/anjuta-glade-plugin.png",
+	BEGIN_REGISTER_ICON (plugin);
+	REGISTER_ICON ("anjuta-glade-plugin-48.png",
 				   "glade-plugin-icon");
+	END_REGISTER_ICON;
 }
 
 static void
 on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 				 AnjutaSession *session, GladePlugin *plugin)
 {
-	GList *files;
-	GtkTreeModel *model;
+	GList *files, *docwids, *node;
+/*	GtkTreeModel *model;
 	GtkTreeIter iter;
+*/
+	IAnjutaDocumentManager *docman;
 	
 	if (phase != ANJUTA_SESSION_PHASE_NORMAL)
 		return;
 	
+	docman = anjuta_shell_get_interface (ANJUTA_PLUGIN(plugin)->shell,
+										 IAnjutaDocumentManager, NULL);
+	docwids = ianjuta_document_manager_get_doc_widgets (docman, NULL);
+	if (docwids)
+	{
+		files = anjuta_session_get_string_list (session, "File Loader", "Files");
+		if (files)
+			files = g_list_reverse (files);
+		for (node = docwids; node != NULL; node = g_list_next (node))
+		{
+			if (ANJUTA_IS_DESIGN_DOCUMENT (node->data))
+			{
+				gchar *uri;
+				uri = ianjuta_file_get_uri (IANJUTA_FILE (node->data), NULL);
+				if (uri != NULL)
+				{
+					files = g_list_prepend (files, uri);
+					/* uri is not freed here */
+				}
+			}
+		}
+		g_list_free (docwids);
+		if (files)
+		{
+			files = g_list_reverse (files);
+			anjuta_session_set_string_list (session, "File Loader", "Files", files);
+			g_list_foreach (files, (GFunc)g_free, NULL);
+			g_list_free (files);
+		}
+	}
+/*
 	files = anjuta_session_get_string_list (session, "File Loader", "Files");
-	files = g_list_reverse (files);
+	if (files)
+		files = g_list_reverse (files);
 	
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (plugin->priv->projects_combo));
 	if (gtk_tree_model_get_iter_first (model, &iter))
 	{
 		do
 		{
+#if (GLADEUI_VERSION >= 330)
+			const gchar *ppath;
+#endif
 			gchar *uri;
 			GladeProject *project;
 			gtk_tree_model_get (model, &iter, PROJECT_COL, &project, -1);
 #if (GLADEUI_VERSION >= 330)
-			if (glade_project_get_path(project))
+			ppath = glade_project_get_path (project);
+			if (ppath)
 #else
 			if (project->path)
 #endif
 			{
+				uri = gnome_vfs_get_uri_from_local_path (
 #if (GLADEUI_VERSION >= 330)
-				uri = gnome_vfs_get_uri_from_local_path
-							(glade_project_get_path(project));
+					ppath);
 #else
-				uri = gnome_vfs_get_uri_from_local_path (project->path);
+					project->path);
 #endif
 				if (uri)
+				{
+					/ * FIXME only log file if it's still open in docman * /
 					files = g_list_prepend (files, uri);
-				/* URI is not freed here */
+					/ * uri is not freed here * /
+				}
 			}
 		}
 		while (gtk_tree_model_iter_next (model, &iter));
 	}
-	files = g_list_reverse (files);
-	anjuta_session_set_string_list (session, "File Loader", "Files", files);
-	g_list_foreach (files, (GFunc)g_free, NULL);
-	g_list_free (files);
+
+	if (files)
+	{
+		files = g_list_reverse (files);
+		anjuta_session_set_string_list (session, "File Loader", "Files", files);
+		g_list_foreach (files, (GFunc)g_free, NULL);
+		g_list_free (files);
+	}
+*/
 }
 
-/* FIXME: Glade does not allow destroying its (singleton) widgets.
- * Make sure they are removed when the application is destroyed
- * without the plugin unloaded.
- */
-static void
-on_shell_destroy (AnjutaShell *shell, GladePlugin *glade_plugin)
-{
-	GtkWidget *wid;
-	GtkWidget *parent;
-	
-	/* Remove widgets before the container destroyes them */
-	wid = GTK_WIDGET (glade_app_get_palette ());
-	parent = gtk_widget_get_parent (wid);
-	gtk_container_remove (GTK_CONTAINER (parent), wid);
-	
-	wid = GTK_WIDGET (glade_app_get_editor ());
-	parent = gtk_widget_get_parent (wid);
-	gtk_container_remove (GTK_CONTAINER (parent), wid);
-	
-	wid = GTK_WIDGET (glade_plugin->priv->view_box);
-	parent = gtk_widget_get_parent (wid);
-	gtk_container_remove (GTK_CONTAINER (parent), wid);
-}
-
-#if (GLADEUI_VERSION > 302)
 static void
 glade_plugin_add_project (GladePlugin *glade_plugin, GladeProject *project)
 {
 	GtkWidget *view;
 	GladePluginPriv *priv;
+	IAnjutaDocumentManager* docman = 
+		anjuta_shell_get_interface(ANJUTA_PLUGIN(glade_plugin)->shell,
+								   IAnjutaDocumentManager, NULL);
 	
  	g_return_if_fail (GLADE_IS_PROJECT (project));
  	
 	priv = glade_plugin->priv;
- 	view = glade_design_view_new (project);	
+ 	view = anjuta_design_document_new(glade_plugin, project);
+	g_signal_connect (G_OBJECT(view), "destroy",
+					  G_CALLBACK (on_document_destroy), glade_plugin);
 	gtk_widget_show (view);
 	g_object_set_data (G_OBJECT (project), "design_view", view);
-	glade_app_add_project (project);
-
-	gtk_notebook_append_page (GTK_NOTEBOOK (priv->design_notebook), GTK_WIDGET (view), NULL);
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->design_notebook), -1);	
+	ianjuta_document_manager_add_document(docman, IANJUTA_DOCUMENT(view), NULL);
 }
-#else
-static void
-glade_plugin_add_project (GladePlugin *glade_plugin, GladeProject *project)
-{
-	glade_app_add_project (project);
-}
-#endif
 
 #if (GLADEUI_VERSION >= 330)
 static void
@@ -752,6 +384,17 @@ inspector_item_activated_cb (GladeInspector     *inspector,
 	
 	g_list_free (item);
 }
+
+static void
+on_glade_resource_removed (GladeProject *project, GladePlugin *plugin)
+{
+}
+
+static void
+on_glade_resource_updated (GladeProject *project, GladePlugin *plugin)
+{
+}
+
 #endif
 
 static gboolean
@@ -796,20 +439,17 @@ activate_plugin (AnjutaPlugin *plugin)
 		g_object_unref (G_OBJECT (store));
 		gtk_box_pack_start (GTK_BOX (priv->view_box), priv->projects_combo,
 							FALSE, FALSE, 0);
-#if (GLADEUI_VERSION > 302)
-#  if (GLADEUI_VERSION >= 330)
+
+#if (GLADEUI_VERSION >= 330)
         priv->inspector = glade_inspector_new ();
         
         g_signal_connect (priv->inspector, "item-activated",
         				  G_CALLBACK (inspector_item_activated_cb),
         				  plugin);
-#  else
-		priv->inspector = glade_project_view_new ();
-#  endif
 #else
-		priv->inspector = glade_project_view_new (GLADE_PROJECT_VIEW_TREE);
+		priv->inspector = glade_project_view_new ();
 #endif
-
+		
 #if (GLADEUI_VERSION < 330)
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->inspector),
 										GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -823,17 +463,11 @@ activate_plugin (AnjutaPlugin *plugin)
 		gtk_notebook_set_scrollable (GTK_NOTEBOOK (glade_app_get_editor ()->notebook),
 									 TRUE);
 		gtk_notebook_popup_enable (GTK_NOTEBOOK (glade_app_get_editor ()->notebook));
-
-		
-#if (GLADEUI_VERSION > 302)
-		/* Create design_notebook */
-		priv->design_notebook = gtk_notebook_new ();
-		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (priv->design_notebook), FALSE);
-		gtk_notebook_set_show_border (GTK_NOTEBOOK (priv->design_notebook), FALSE);
-		gtk_widget_show (priv->design_notebook);
-#endif
 	}
 	
+	g_signal_connect(G_OBJECT(plugin->shell), "destroy",
+					 G_CALLBACK(on_shell_destroy), plugin);
+	
 	g_signal_connect (G_OBJECT (priv->projects_combo), "changed",
 					  G_CALLBACK (on_glade_project_changed), plugin);
 	g_signal_connect (G_OBJECT (priv->gpw), "update-ui",
@@ -842,29 +476,13 @@ activate_plugin (AnjutaPlugin *plugin)
 	g_signal_connect(G_OBJECT(glade_app_get_editor()), "gtk-doc-search",
 					 G_CALLBACK(on_api_help), plugin);
 	
-	g_signal_connect(G_OBJECT(plugin->shell), "destroy",
-					 G_CALLBACK(on_shell_destroy), plugin);
-	
-	/* Add action group */
-	priv->action_group = 
-		anjuta_ui_add_action_group_entries (ui, "ActionGroupGlade",
-											_("Glade operations"),
-											actions,
-											G_N_ELEMENTS (actions),
-											GETTEXT_PACKAGE, TRUE,
-											glade_plugin);
-	/* Add UI */
-	priv->uiid = anjuta_ui_merge (ui, UI_FILE);
-	
 	/* FIXME: Glade doesn't want to die these widget, so
 	 * hold a permenent refs on them
 	 */
 	g_object_ref (glade_app_get_palette ());
 	g_object_ref (glade_app_get_editor ());
 	g_object_ref (priv->view_box);
-#if (GLADEUI_VERSION > 302)
-	g_object_ref (priv->design_notebook);
-#endif
+
 	gtk_widget_show (GTK_WIDGET (glade_app_get_palette ()));
 	gtk_widget_show (GTK_WIDGET (glade_app_get_editor ()));
 	
@@ -884,16 +502,16 @@ activate_plugin (AnjutaPlugin *plugin)
 							 "AnjutaGladeEditor", _("Properties"),
 							 "glade-plugin-icon",
 							 ANJUTA_SHELL_PLACEMENT_CENTER, NULL);
-#if (GLADEUI_VERSION > 302)
-	anjuta_shell_add_widget (ANJUTA_PLUGIN (plugin)->shell,
-							 GTK_WIDGET (priv->design_notebook),
-							 "AnjutaGladeDesigner", _("Designer"),
-							 "glade-plugin-icon",
-							 ANJUTA_SHELL_PLACEMENT_CENTER, NULL);
-#endif
 	/* Connect to save session */
 	g_signal_connect (G_OBJECT (plugin->shell), "save_session",
 					  G_CALLBACK (on_session_save), plugin);
+	
+	/* Watch documents */
+	glade_plugin->priv->editor_watch_id = 
+		anjuta_plugin_add_watch (plugin, "document_manager_current_editor",
+								 value_added_current_editor,
+								 value_removed_current_editor, NULL);
+	
 	return TRUE;
 }
 
@@ -903,12 +521,14 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	GladePluginPriv *priv;
 	
 	priv = ANJUTA_PLUGIN_GLADE (plugin)->priv;
-	
-	AnjutaUI *ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	
+		
 	DEBUG_PRINT ("GladePlugin: Dectivating Glade plugin...");
 	
 	/* Disconnect signals */
+	g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->shell),
+										  G_CALLBACK (on_shell_destroy),
+										  plugin);
+	
 	g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->shell),
 										  G_CALLBACK (on_session_save), plugin);
 	
@@ -921,10 +541,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	
 	g_signal_handlers_disconnect_by_func (G_OBJECT(glade_app_get_editor()),
 										  G_CALLBACK(on_api_help), plugin);
-	
-	g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->shell),
-										  G_CALLBACK (on_shell_destroy),
-										  plugin);
+
 	/* Remove widgets */
 	anjuta_shell_remove_widget (plugin->shell,
 								GTK_WIDGET (glade_app_get_palette ()),
@@ -935,18 +552,6 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	anjuta_shell_remove_widget (plugin->shell,
 								GTK_WIDGET (priv->view_box),
 								NULL);
-#if (GLADEUI_VERSION > 302)
-	anjuta_shell_remove_widget (plugin->shell,
-								GTK_WIDGET (priv->design_notebook),
-								NULL);
-#endif
-	
-	/* Remove UI */
-	anjuta_ui_unmerge (ui, priv->uiid);
-	
-	/* Remove action group */
-	anjuta_ui_remove_action_group (ui, priv->action_group);
-	
 	/* FIXME: Don't destroy glade, since it doesn't want to */
 	/* g_object_unref (G_OBJECT (priv->gpw)); */
 	/* priv->gpw = NULL */
@@ -964,7 +569,7 @@ glade_plugin_dispose (GObject *obj)
 	/* GladePlugin *plugin = ANJUTA_PLUGIN_GLADE (obj); */
 	
 	/* FIXME: Glade widgets should be destroyed */
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void
@@ -972,7 +577,7 @@ glade_plugin_finalize (GObject *obj)
 {
 	GladePlugin *plugin = ANJUTA_PLUGIN_GLADE (obj);
 	g_free (plugin->priv);
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -983,6 +588,7 @@ glade_plugin_instance_init (GObject *obj)
 	
 	plugin->priv = (GladePluginPriv *) g_new0 (GladePluginPriv, 1);
 	priv = plugin->priv;
+	priv->destroying = FALSE;
 	
 	DEBUG_PRINT ("Intializing Glade plugin");
 }
@@ -997,7 +603,20 @@ glade_plugin_class_init (GObjectClass *klass)
 	plugin_class->activate = activate_plugin;
 	plugin_class->deactivate = deactivate_plugin;
 	klass->dispose = glade_plugin_dispose;
-	klass->dispose = glade_plugin_finalize;
+	klass->finalize = glade_plugin_finalize;
+}
+
+gchar* glade_get_filename(GladePlugin *plugin)
+{
+#if (GLADEUI_VERSION >= 330)
+	return glade_project_get_name(glade_app_get_project());
+#else
+	GladeProject* project = glade_app_get_project();
+	if (project)
+		return project->name;
+	else
+		return NULL;
+#endif
 }
 
 static void
@@ -1008,6 +627,9 @@ ifile_open (IAnjutaFile *ifile, const gchar *uri, GError **err)
 	GtkListStore *store;
 	GtkTreeIter iter;
 	gchar *filename;
+	IAnjutaDocumentManager* docman;
+	GList* docwids, *node;
+	GList *glade_obj_node;
 	
 	g_return_if_fail (uri != NULL);
 	
@@ -1020,8 +642,47 @@ ifile_open (IAnjutaFile *ifile, const gchar *uri, GError **err)
 								    _("Not local file: %s"), uri);
 		return;
 	}
+	
+	docman = anjuta_shell_get_interface(ANJUTA_PLUGIN(ifile)->shell, IAnjutaDocumentManager,
+										NULL);
+	docwids = ianjuta_document_manager_get_doc_widgets (docman, NULL);
+	if (docwids)
+	{
+		for (node = docwids; node != NULL; node = g_list_next (node))
+		{
+			if (ANJUTA_IS_DESIGN_DOCUMENT (node->data))
+			{
+				gchar *cur_uri;
+				cur_uri = ianjuta_file_get_uri (IANJUTA_FILE (node->data), NULL);
+				if (cur_uri)
+				{
+					DEBUG_PRINT("%s = %s", uri, cur_uri);
+					if (g_str_equal (uri, cur_uri))
+					{
+						ianjuta_document_manager_set_current_document (docman,
+							IANJUTA_DOCUMENT (node->data), NULL);
+						g_free (cur_uri);
+						g_list_free (docwids);
+						return;
+					}
+					g_free (cur_uri);
+				}
+			}
+		}
+		g_list_free (docwids);
+	}
+	
 #if (GLADEUI_VERSION >= 330)
 	project = glade_project_load (filename);
+	if (project)
+	{
+		g_signal_connect (project, "resource-removed",
+						  G_CALLBACK (on_glade_resource_removed),
+						  ANJUTA_PLUGIN_GLADE (ifile));
+		g_signal_connect (project, "resource-updated",
+						  G_CALLBACK (on_glade_resource_updated),
+						  ANJUTA_PLUGIN_GLADE (ifile));
+	}
 #else
 	project = glade_project_open (filename);
 #endif
@@ -1029,7 +690,7 @@ ifile_open (IAnjutaFile *ifile, const gchar *uri, GError **err)
 	if (!project)
 	{
 		anjuta_util_dialog_warning (GTK_WINDOW (ANJUTA_PLUGIN (ifile)->shell),
-								    _("Could not open: %s"), uri);
+								    _("Could not open %s"), uri);
 		return;
 	}
 	store = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (priv->projects_combo)));
@@ -1042,13 +703,43 @@ ifile_open (IAnjutaFile *ifile, const gchar *uri, GError **err)
 						PROJECT_COL, project, -1);
 #endif
 	glade_plugin_add_project (ANJUTA_PLUGIN_GLADE (ifile), project);
+
+	/* Select the first window in the project */
+	for (glade_obj_node = (GList *) glade_project_get_objects (project);
+		 glade_obj_node != NULL;
+		 glade_obj_node = g_list_next (glade_obj_node))
+	{
+		GObject *glade_obj = G_OBJECT (glade_obj_node->data);
+		if (GTK_IS_WINDOW (glade_obj))
+			glade_widget_show (glade_widget_get_from_gobject (glade_obj));
+		break;
+	}
 	anjuta_shell_present_widget (ANJUTA_PLUGIN (ifile)->shell, priv->view_box, NULL);
 }
 
+static gchar*
+ifile_get_uri (IAnjutaFile* file, GError** e)
+{
+#if (GLADEUI_VERSION >= 330)
+	const gchar* path = glade_project_get_path(glade_app_get_project());
+	if (path != NULL)
+		return gnome_vfs_get_uri_from_local_path(path);
+	else
+		return NULL;
+#else
+	GladeProject* project = glade_app_get_project();
+	if (project && project->path)
+		return project->path;
+	else
+		return NULL;
+#endif
+}
+
 static void
 ifile_iface_init(IAnjutaFileIface *iface)
 {
 	iface->open = ifile_open;
+	iface->get_uri = ifile_get_uri;
 }
 
 static void
@@ -1063,6 +754,13 @@ iwizard_activate (IAnjutaWizard *iwizard, GError **err)
 
 #if (GLADEUI_VERSION >= 330)
 	project = glade_project_new ();
+	
+	g_signal_connect (project, "resource-removed",
+					  G_CALLBACK (on_glade_resource_removed),
+					  ANJUTA_PLUGIN_GLADE (iwizard));
+	g_signal_connect (project, "resource-updated",
+					  G_CALLBACK (on_glade_resource_updated),
+					  ANJUTA_PLUGIN_GLADE (iwizard));
 #else
 	project = glade_project_new (TRUE);
 #endif
