@@ -54,6 +54,7 @@
 #define PREF_INDENT_TAB_INDENTS "language.cpp.indent.tab.indents"
 #define PREF_INDENT_STATEMENT_SIZE "language.cpp.indent.statement.size"
 #define PREF_INDENT_BRACE_SIZE "language.cpp.indent.brace.size"
+#define PREF_BRACE_AUTOCOMPLETION "language.cpp.brace.autocompletion"
 
 #define TAB_SIZE (ianjuta_editor_get_tabsize (editor, NULL))
 
@@ -135,7 +136,7 @@ skip_iter_to_newline_tail (IAnjutaIterable *iter, gchar ch)
 static gint
 get_line_indentation (IAnjutaEditor *editor, gint line_num)
 {
-	gint line_begin, line_end;
+	IAnjutaIterable *line_begin, *line_end;
 	gchar *line_string, *idx;
 	gint line_indent = 0;
 	
@@ -145,11 +146,18 @@ get_line_indentation (IAnjutaEditor *editor, gint line_num)
 	DEBUG_PRINT ("%s: line begin = %d, line end = %d", __FUNCTION__,
 				 line_begin, line_end);
 	*/
-	if (line_begin == line_end)
+	if (ianjuta_iterable_compare (line_begin, line_end, NULL) == 0)
+	{
+		g_object_unref (line_begin);
+		g_object_unref (line_end);
 		return 0;
+	}
+	
+	line_string = ianjuta_editor_get_text (editor, line_begin, line_end,
+												NULL);
+	g_object_unref (line_begin);
+	g_object_unref (line_end);
 	
-	line_string = ianjuta_editor_get_text (editor, line_begin,
-										   line_end - line_begin, NULL);
 	/* DEBUG_PRINT ("line_string = '%s'", line_string); */
 	
 	if (!line_string)
@@ -165,7 +173,7 @@ get_line_indentation (IAnjutaEditor *editor, gint line_num)
 		else
 			line_indent++;
 		idx++; /* Since we are looking for first non-space char, simple
-	            * increment of the chars would do */
+	            * increment of the utf8 chars would do */
 	}
 	g_free (line_string);
 	return line_indent;
@@ -274,9 +282,7 @@ skip_iter_to_previous_logical_line (IAnjutaEditor *editor,
 	{
 		/*
 		DEBUG_PRINT ("Line %d is continuation line .. Skipping",
-					 ianjuta_editor_get_line_from_position
-						(editor, ianjuta_iterable_get_position (iter, NULL),
-						 NULL));
+					 ianjuta_editor_get_line_from_position (editor, iter, NULL));
 		*/
 		found = skip_iter_to_previous_line (editor, iter);
 		if (!found)
@@ -284,17 +290,13 @@ skip_iter_to_previous_logical_line (IAnjutaEditor *editor,
 	}
 	/*
 	DEBUG_PRINT ("Line %d is *not* continuation line .. Breaking",
-				 ianjuta_editor_get_line_from_position
-					(editor, ianjuta_iterable_get_position (iter, NULL),
-					 NULL));
+				 ianjuta_editor_get_line_from_position (editor, iter, NULL));
 	*/
 	if (found)
 		found = skip_iter_to_previous_line (editor, iter);
 	/*
 	DEBUG_PRINT ("Line %d is next logical line",
-				 ianjuta_editor_get_line_from_position
-					(editor, ianjuta_iterable_get_position (iter, NULL),
-					 NULL));
+				 ianjuta_editor_get_line_from_position (editor, iter, NULL));
 	*/
 	return found;
 }
@@ -372,8 +374,7 @@ skip_preprocessor_lines (IAnjutaEditor *editor, IAnjutaIterable *iter)
 				/*
 				DEBUG_PRINT ("Line %d is preprocessor line .. Skipping",
 							 ianjuta_editor_get_line_from_position
-								(editor, ianjuta_iterable_get_position (new_iter, NULL),
-								 NULL));
+							 (editor, new_iter, NULL));
 				*/
 				break;
 			}
@@ -396,8 +397,7 @@ skip_preprocessor_lines (IAnjutaEditor *editor, IAnjutaIterable *iter)
 			/*
 			DEBUG_PRINT ("Line %d is *not* preprocessor line .. Breaking",
 						 ianjuta_editor_get_line_from_position
-							(editor, ianjuta_iterable_get_position (new_iter, NULL),
-							 NULL));
+							(editor, new_iter, NULL));
 			*/
 			break;
 		}
@@ -542,10 +542,10 @@ extract_mode_line (const gchar *comment_text, gboolean* vim)
 		begin_modeline += 3;
 		end_modeline = strstr (begin_modeline, "-*-");
 		if (end_modeline)
-    {
-      *vim = FALSE;
-			return g_strndup (begin_modeline, end_modeline - begin_modeline);
-    }
+		{
+		  *vim = FALSE;
+				return g_strndup (begin_modeline, end_modeline - begin_modeline);
+		}
 	}
 	/* Search for vim-like modelines */
 	begin_modeline = strstr (comment_text, "vim:set");
@@ -556,7 +556,7 @@ extract_mode_line (const gchar *comment_text, gboolean* vim)
 		/* Check for escape characters */
 		while (end_modeline)
 		{
-			 if ((end_modeline - 1) != "\\")
+			 if (!g_str_equal ((end_modeline - 1), "\\"))
 				break;
 			end_modeline++;
 			end_modeline = strstr (end_modeline, ":");
@@ -564,7 +564,7 @@ extract_mode_line (const gchar *comment_text, gboolean* vim)
 		if (end_modeline)
 		{
 			gchar* vim_modeline = g_strndup (begin_modeline, end_modeline - begin_modeline);
-      *vim = TRUE;
+			*vim = TRUE;
 			return vim_modeline;
 		}
 	}
@@ -592,8 +592,8 @@ initialize_indentation_params (CppJavaPlugin *plugin)
 	
 	/* Find the first comment text in the buffer */
 	comment_text = g_string_new (NULL);
-	iter = ianjuta_editor_get_cell_iter (IANJUTA_EDITOR (plugin->current_editor),
-										 0, NULL);
+	iter = ianjuta_editor_get_start_position (IANJUTA_EDITOR (plugin->current_editor),
+											  NULL);
 	do
 	{
 		gboolean shift_buffer = TRUE;
@@ -672,14 +672,12 @@ initialize_indentation_params (CppJavaPlugin *plugin)
 static gint
 set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
 {
-	gint line_begin, line_end, indent_position;
-	gint current_pos;
-	gchar *line_string, *idx;
-	gint nchars = 0, nchars_removed = 0;
+	IAnjutaIterable *line_begin, *line_end, *indent_position;
+	IAnjutaIterable *current_pos;
+	gint carat_offset, nchars = 0, nchars_removed = 0;
 	gchar *old_indent_string = NULL, *indent_string = NULL;
 	
 	/* DEBUG_PRINT ("In %s()", __FUNCTION__); */
-	current_pos = ianjuta_editor_get_position (editor, NULL);
 	line_begin = ianjuta_editor_get_line_begin_position (editor, line_num, NULL);
 	line_end = ianjuta_editor_get_line_end_position (editor, line_num, NULL);
 	
@@ -687,45 +685,54 @@ set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
 	DEBUG_PRINT ("line begin = %d, line end = %d, current_pos = %d",
 				 line_begin, line_end, current_pos);
 	*/
-	indent_position = line_begin;
+	indent_position = ianjuta_iterable_clone (line_begin, NULL);
 	
-	if (line_end > line_begin)
+	if (ianjuta_iterable_compare (line_end, line_begin, NULL) > 0)
 	{
-		line_string = ianjuta_editor_get_text (editor, line_begin,
-											   line_end - line_begin,
-											   NULL);
+		gchar *idx;
+		gchar *line_string = ianjuta_editor_get_text (editor, line_begin,
+														   line_end, NULL);
 		
-		/* DEBUG_PRINT ("line_string = '%s'", line_string); */
+		DEBUG_PRINT ("line_string = '%s'", line_string);
 		if (line_string)
 		{
 			idx = line_string;
 			
 			/* Find first non-white space */
 			while (*idx != '\0' && isspace (*idx))
-				idx++; /* Since we are looking for first non-space char, simple
-						* increment of the chars would do */
-			indent_position = line_begin + (idx - line_string);
+			{
+				idx = g_utf8_find_next_char (idx, NULL);
+				ianjuta_iterable_next (indent_position, NULL);
+			}
 			g_free (line_string);
 		}
 	}
+	/* Indent iter defined at this point, Identify how much is current
+	 * position is beyound this point. We need to restore it later after
+	 * indentation
+	*/
+	current_pos = ianjuta_editor_get_position (editor, NULL);
+	carat_offset = ianjuta_iterable_diff (indent_position, current_pos, NULL);
+	DEBUG_PRINT ("carat offset is = %d", carat_offset);
 	
 	/* Set new indentation */
 	if (indentation > 0)
 	{
-		indent_string = get_line_indentation_string (editor,
-													 indentation);
-		nchars = strlen (indent_string);
+		indent_string = get_line_indentation_string (editor, indentation);
+		nchars = g_utf8_strlen (indent_string, -1);
 		
 		/* Only indent if there is something to indent with */
 		if (indent_string)
 		{
 			/* Get existing indentation */
-			if (indent_position > line_begin)
+			if (ianjuta_iterable_compare (indent_position, line_begin, NULL) > 0)
 			{
-				old_indent_string = ianjuta_editor_get_text (editor, line_begin,
-															 indent_position - line_begin,
-															 NULL);
-				nchars_removed = strlen (old_indent_string);
+				old_indent_string =
+					ianjuta_editor_get_text (editor, line_begin,
+												  indent_position, NULL);
+				
+				DEBUG_PRINT ("old_indent_string = '%s'", old_indent_string);
+				nchars_removed = g_utf8_strlen (old_indent_string, -1);
 			}
 			
 			/* Only indent if there was no indentation before or old
@@ -737,7 +744,7 @@ set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
 				/* Remove the old indentation string, if there is any */
 				if (old_indent_string)
 					ianjuta_editor_erase (editor, line_begin,
-										  indent_position - line_begin, NULL);
+										  indent_position, NULL);
 				
 				/* Insert the new indentation string */
 				ianjuta_editor_insert (editor, line_begin,
@@ -752,41 +759,48 @@ set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
 	if (indentation == 0)
 	{
 		/* Get existing indentation */
-		if (indent_position > line_begin)
+		if (ianjuta_iterable_compare (indent_position, line_begin, NULL) > 0)
 		{
-			old_indent_string = ianjuta_editor_get_text (editor, line_begin,
-														 indent_position - line_begin,
-														 NULL);
-			nchars_removed = strlen (old_indent_string);
+			old_indent_string =
+				ianjuta_editor_get_text (editor, line_begin,
+											  indent_position, NULL);
+			nchars_removed = g_utf8_strlen (old_indent_string, -1);
 		}
 		if (old_indent_string)
-			ianjuta_editor_erase (editor, line_begin,
-								  indent_position - line_begin, NULL);
-				
+			ianjuta_editor_erase (editor, line_begin, indent_position, NULL);
 	}
 	
 	/* Restore current position */
-	if (current_pos >= indent_position)
+	if (carat_offset >= 0)
 	{
 		/* If the cursor was not before the first non-space character in
 		 * the line, restore it's position after indentation.
 		 */
-		gint delta_position = nchars - nchars_removed;
-		ianjuta_editor_goto_position (editor, current_pos + delta_position,
-									  NULL);
-		/*
-		DEBUG_PRINT ("Restored position to %d", current_pos + delta_position);
-		*/
+		gint i;
+		IAnjutaIterable *pos = ianjuta_editor_get_line_begin_position (editor, line_num, NULL);
+		for (i = 0; i < nchars + carat_offset; i++)
+			ianjuta_iterable_next (pos, NULL);
+		ianjuta_editor_goto_position (editor, pos, NULL);
+		g_object_unref (pos);
 	}
-	else if (current_pos >= line_begin)
+	else /* cursor_offset < 0 */
 	{
 		/* If the cursor was somewhere in the old indentation spaces,
 		 * home the cursor to first non-space character in the line (or
 		 * end of line if there is no non-space characters in the line.
 		 */
-		ianjuta_editor_goto_position (editor, current_pos + nchars,
-									  NULL);
+		gint i;
+		IAnjutaIterable *pos = ianjuta_editor_get_line_begin_position (editor, line_num, NULL);
+		for (i = 0; i < nchars; i++)
+			ianjuta_iterable_next (pos, NULL);
+		ianjuta_editor_goto_position (editor, pos, NULL);
+		g_object_unref (pos);
 	}
+
+	g_object_unref (current_pos);
+	g_object_unref (indent_position);
+	g_object_unref (line_begin);
+	g_object_unref (line_end);
 	
 	g_free (old_indent_string);
 	g_free (indent_string);
@@ -802,15 +816,17 @@ static gint
 get_line_indentation_base (CppJavaPlugin *plugin,
 						   IAnjutaEditor *editor,
 						   gint line_num,
-						   gint *incomplete_statement)
+						   gint *incomplete_statement,
+						   gboolean *colon_indent)
 {
 	IAnjutaIterable *iter;
 	gchar point_ch;
-	gint position;
 	gint line_indent = 0;
+	gint extra_indent = 0;
 	gboolean looking_at_just_next_line = TRUE;
 	gboolean current_line_is_preprocessor = FALSE;
 	gboolean current_line_is_continuation = FALSE;
+	gboolean line_checked_for_comment = FALSE;
 	
 	*incomplete_statement = -1;
 	
@@ -819,9 +835,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 	
 	/* DEBUG_PRINT ("In %s()", __FUNCTION__); */
 	
-	position = ianjuta_editor_get_line_begin_position (editor, line_num,
-													   NULL);
-	iter = ianjuta_editor_get_cell_iter (editor, position, NULL);
+	iter = ianjuta_editor_get_line_begin_position (editor, line_num, NULL);
 	
 	current_line_is_preprocessor = line_is_preprocessor (editor, iter);
 	current_line_is_continuation = line_is_continuation (editor, iter);
@@ -848,23 +862,163 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 	
 	while (ianjuta_iterable_previous (iter, NULL))
 	{
-		/* Skip comments and strings */
+		/* Skip strings */
 		IAnjutaEditorAttribute attrib =
 			ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL (iter), NULL);
-		if (attrib == IANJUTA_EDITOR_COMMENT || attrib == IANJUTA_EDITOR_STRING)
+		if (attrib == IANJUTA_EDITOR_STRING)
 			continue;
 		
 		point_ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (iter), 0,
 												 NULL);
 
 		/* DEBUG_PRINT("point_ch = %c", point_ch); */
+		
+		/* Check for line comment comment */
+		if (!line_checked_for_comment && !isspace(point_ch))
+		{
+			gboolean comment = FALSE;
+			IAnjutaIterable* new_iter = ianjuta_iterable_clone (iter, NULL);
+			do
+			{
+				gchar c;
+				c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (new_iter), 0,
+												  NULL);
+				if (iter_is_newline (new_iter, c))
+				{
+					line_checked_for_comment = TRUE;
+					break;
+				}
+				if (c == '/')
+				{
+					IAnjutaIterable* tmp_iter = ianjuta_iterable_clone (new_iter, NULL);
+					if (!ianjuta_iterable_previous (tmp_iter, NULL))
+					{
+						g_object_unref (tmp_iter);
+						break;
+					}
+					c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (tmp_iter), 0,
+													  NULL);
+					if (c == '/')
+					{
+						/* is a line comment, skip until begin of comment */
+						comment = TRUE;
+						g_object_unref (tmp_iter);
+						break;
+					}
+					g_object_unref (tmp_iter);
+				}
+			} while (ianjuta_iterable_previous (new_iter, NULL));
+			if (comment)
+			{
+				ianjuta_iterable_set_position (iter,
+											   ianjuta_iterable_get_position (new_iter, NULL) - 1, 
+											   NULL);
+				g_object_unref (new_iter);
+				continue;
+			}
+			g_object_unref (new_iter);
+		}
+		/* Check if we are inside a comment */
+		if (point_ch == '/' || point_ch == '*')
+		{
+			gboolean comment = FALSE;
+			gboolean comment_end = FALSE;
+			IAnjutaIterable* new_iter = ianjuta_iterable_clone (iter, NULL);
+			do
+			{
+				gchar c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL(new_iter),
+												  0, NULL);
+				if (!comment_end && iter_is_newline (new_iter, c))
+				{
+					break;
+				}
+				if (c == '*')
+				{
+					IAnjutaIterable* prev = ianjuta_iterable_clone (new_iter, NULL);
+					IAnjutaIterable* next = ianjuta_iterable_clone (new_iter, NULL);					
+					ianjuta_iterable_previous (prev, NULL);
+					ianjuta_iterable_next (next, NULL);
+					gchar prev_c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (prev), 0,
+													  NULL);
+					gchar next_c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (next), 0,
+													  NULL);					
+					if (prev_c == '/')
+					{		 
+						/* starts comment */
+						comment = TRUE;
+						if (!comment_end)
+						{
+							extra_indent++;
+							/* In the middle of a comment we can't know
+						     * if the statement is incomplete
+							 */
+							*incomplete_statement = -1;
+							/* ":" have to be ignored inside comments */
+							if (*colon_indent)
+							{
+								*colon_indent = FALSE;
+								extra_indent -= INDENT_SIZE;
+							}
+						}
+						g_object_unref (prev);
+						g_object_unref (next);
+						break;
+						
+					}
+					else if (next_c == '/')
+					{
+						/* ends comment: */
+						comment_end = TRUE;
+						g_object_unref (prev);
+						g_object_unref (next);
+						continue;
+					}
+					/* Possibly continued comment */
+					else if (isspace(prev_c))
+					{
+						gboolean possible_comment = FALSE;
+						while (ianjuta_iterable_previous (prev, NULL))
+						{
+							prev_c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (prev), 0,
+															  NULL);
+							if (!isspace(prev_c))
+								break;
+							if (iter_is_newline (prev, prev_c))
+							{
+								possible_comment = TRUE;
+								break;
+							}
+						}
+						if (possible_comment)
+						{
+							ianjuta_iterable_set_position (new_iter, 
+														   ianjuta_iterable_get_position (prev, NULL),
+														   NULL);
+							g_object_unref (prev);
+							g_object_unref (next);
+							continue;
+						}
+					}
+					g_object_unref (prev);
+					g_object_unref (next);
+				}
+			} while (ianjuta_iterable_previous (new_iter, NULL));
+			if (comment)
+			{
+				ianjuta_iterable_set_position (iter,
+											   ianjuta_iterable_get_position (new_iter, NULL) - 1, 
+											   NULL);
+				g_object_unref (new_iter);
+				continue;
+			}
+			g_object_unref (new_iter);
+		}
 		if (point_ch == ')' || point_ch == ']' || point_ch == '}')
 		{
 			gint line_saved;
 			
-			line_saved = ianjuta_editor_get_line_from_position (editor,
-								ianjuta_iterable_get_position (iter, NULL),
-																	NULL);
+			line_saved = ianjuta_editor_get_line_from_position (editor, iter,
+																NULL);
 			
 			/* If we encounter a block-end before anything else, the
 			 * statement could hardly be incomplte.
@@ -878,6 +1032,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			if (point_ch == '}' && get_line_indentation (editor, line_saved) <= 0)
 			{
 				line_indent = 0;
+				line_indent += extra_indent;
 				break;
 			}
 			
@@ -885,17 +1040,18 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			if (!cpp_java_util_jump_to_matching_brace (iter, point_ch, -1))
 			{
 				line_indent = get_line_indentation (editor, line_saved);
+				line_indent += extra_indent;
 				break;
 			}
-		}
+			}
 		else if (point_ch == '{')
 		{
-			gint line_for_indent = ianjuta_editor_get_line_from_position (editor,
-								ianjuta_iterable_get_position (iter, NULL),
-								NULL);
+			gint line_for_indent =
+				ianjuta_editor_get_line_from_position (editor, iter, NULL);
 			line_indent = get_line_indentation (editor, line_for_indent);
 			/* Increase line indentation */
 			line_indent += INDENT_SIZE;
+			line_indent += extra_indent;
 			
 			/* If we encounter a block-start before anything else, the
 			 * statement could hardly be incomplte.
@@ -923,6 +1079,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 					line_indent++;
 			}
 			line_indent += 1;
+			line_indent += extra_indent;
 			
 			/* Although statement is incomplete at this point, we don't
 			 * set it to incomplete and just leave it to unknown to avaoid
@@ -940,6 +1097,37 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			if (*incomplete_statement == -1)
 				*incomplete_statement = 0;
 		}
+		else if (point_ch == ':' && *colon_indent == FALSE)
+		{
+			/* This is a forward reference, all lines below should have
+			 * increased indentation until the next statement has
+			 * a ':'
+			 */
+			IAnjutaIterable* new_iter = ianjuta_iterable_clone (iter, NULL);
+			gboolean indent = FALSE;		
+			
+			/* Is the last non-whitespace in line */
+			while (ianjuta_iterable_next (new_iter, NULL))
+			{
+				gchar c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (new_iter),
+													   0, NULL);
+				if (!isspace(c))
+					break;
+				if (iter_is_newline (new_iter, c))
+				{
+					indent = TRUE;
+					break;
+				}
+			}
+			g_object_unref (new_iter);
+			if (indent)
+			{
+				*colon_indent = TRUE;
+				extra_indent += INDENT_SIZE;
+				if (*incomplete_statement == -1)
+					*incomplete_statement = 0;
+			}
+		}	
 		else if (iter_is_newline (iter, point_ch))
 		{
 			skip_iter_to_newline_head (iter, point_ch);
@@ -952,15 +1140,15 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 				looking_at_just_next_line)
 			{
 				/*
-				gint line = ianjuta_editor_get_line_from_position
-					(editor, ianjuta_iterable_get_position (iter, NULL), NULL);
+				gint line = ianjuta_editor_get_line_from_position (editor, iter, NULL);
 				line_indent = get_line_indentation (editor, line);
 				*/
 			}
 			looking_at_just_next_line = FALSE;
-		}
+			line_checked_for_comment = FALSE;
+		}		 
 		else if (!isspace (point_ch))
-		{
+		{			
 			/* If we encounter any non-whitespace char before any of the
 			 * statement-complete indicators, the statement is basically
 			 * incomplete
@@ -969,47 +1157,50 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 				*incomplete_statement = 1;
 		}
 	}
+	if (!line_indent && extra_indent)
+	{
+		DEBUG_PRINT ("Adding special indent");
+		line_indent += extra_indent;
+	}
 	g_object_unref (iter);
 	
 	return line_indent;
 }
 
-/* Check if iter is inside comment or string. Begining of comment or string
+/* Check if iter is inside string. Begining of string
  * is not counted as inside.
  */
 static gboolean
-is_iter_inside_comment_or_string (IAnjutaIterable *iter)
+is_iter_inside_string (IAnjutaIterable *iter)
 {
 	IAnjutaEditorAttribute attrib;
 	
 	attrib = ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL (iter),
 												NULL);
-	/* Check if we are *inside* comment or string. Begining of comment
-	 * or string does not count as inside.
+	/* Check if we are *inside* string. Begining
+	 * of string does not count as inside.
 	 */
-	if (attrib == IANJUTA_EDITOR_COMMENT || attrib == IANJUTA_EDITOR_STRING)
+	if (attrib == IANJUTA_EDITOR_STRING)
 	{
 		/* Peek previous attrib and see what it was */
 		if (ianjuta_iterable_previous (iter, NULL))
 		{
 			attrib = ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL
 														(iter),	NULL);
-			if (attrib == IANJUTA_EDITOR_COMMENT ||
-				attrib == IANJUTA_EDITOR_STRING)
+			if (attrib == IANJUTA_EDITOR_STRING)
 			{
-				/* We are inside comment or string */
+				/* We are inside string */
 				return TRUE;
 			}
 			else
 			{
-				/* The comment or string just began, not inside.
+				/* The string just began, not inside.
 				 * Restore iter from the peek
 				 */
 				ianjuta_iterable_next (iter, NULL);
 			}
 		}
-		/* else, there is no previous and so we can't be inside comment
-		 * or string
+		/* else, there is no previous and so we can't be inside string
 		 */
 	}
 	return FALSE;
@@ -1020,23 +1211,47 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 						   gint line)
 {
 	IAnjutaIterable *iter;
+	IAnjutaIterable *end_iter;
 	gint line_indent = 0;
-	gint line_begin;
 	gint incomplete_statement = -1;
+	gboolean colon_indent = FALSE;
 	
-	g_return_val_if_fail (line > 1, 0);
+	g_return_val_if_fail (line > 0, 0);
 	
-	line_begin = ianjuta_editor_get_line_begin_position (editor, line, NULL);
-	iter = ianjuta_editor_get_cell_iter (editor, line_begin, NULL);
+	if (line == 1) /* First line */
+		return 0;
 	
-	if (is_iter_inside_comment_or_string (iter))
+	iter = ianjuta_editor_get_line_begin_position (editor, line, NULL);
+	
+	if (is_iter_inside_string (iter))
 	{
 		line_indent = get_line_indentation (editor, line - 1);
 	}
 	else
 	{
 		line_indent = get_line_indentation_base (plugin, editor, line,
-												 &incomplete_statement);
+												 &incomplete_statement, &colon_indent);
+	}
+	
+	if (colon_indent)
+	{
+		/* If the last non-whitespace character in the line is ":" then
+		 * we remove the extra colon_indent
+		 */
+		end_iter = ianjuta_editor_get_line_end_position (editor, line, NULL);
+		gchar ch;
+		do
+		{
+			ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (end_iter),
+											   0, NULL);
+			if (ch == ':')
+			{
+				line_indent -= INDENT_SIZE;
+				break;
+			}
+		}
+		while (isspace(ch) && ianjuta_iterable_previous (end_iter, NULL));
+		g_object_unref (end_iter);
 	}
 	
 	/* Determine what the first non-white char in the line is */
@@ -1047,7 +1262,7 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 		 * or string does not count as inside. If inside, just align with
 		 * previous indentation.
 		 */
-		if (is_iter_inside_comment_or_string (iter))
+		if (is_iter_inside_string (iter))
 		{
 			line_indent = get_line_indentation (editor, line - 1);
 			break;
@@ -1075,6 +1290,9 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 							 line_indent + BRACE_INDENT);
 				*/
 				line_indent += BRACE_INDENT;
+				/* It looks ugly to add extra indent after case: so remove that */
+				if (colon_indent)
+					line_indent -= INDENT_SIZE;
 			}
 			break;
 		}
@@ -1083,9 +1301,8 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 			ianjuta_iterable_previous (iter, NULL);
 			if (cpp_java_util_jump_to_matching_brace (iter, ch, -1))
 			{
-				gint position = ianjuta_iterable_get_position (iter, NULL);
 				gint line = ianjuta_editor_get_line_from_position (editor,
-																   position,
+																   iter,
 																   NULL);
 				line_indent = get_line_indentation (editor, line);
 			}
@@ -1110,105 +1327,159 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 
 static void
 on_editor_char_inserted_cpp (IAnjutaEditor *editor,
-							 gint insert_pos,
+							 IAnjutaIterable *insert_pos,
 							 gchar ch,
 							 CppJavaPlugin *plugin)
 {
 	IAnjutaEditorAttribute attrib;
 	IAnjutaIterable *iter;
 	gboolean should_auto_indent = FALSE;
+	static GString *stack = NULL;
+
+	iter = ianjuta_iterable_clone (insert_pos, NULL);
 	
-	/* Do nothing if automatic indentation is not enabled */
-	if (!anjuta_preferences_get_int (plugin->prefs, PREF_INDENT_AUTOMATIC))
-		return;
-	
-	/* DEBUG_PRINT ("Char added at position %d: '%c'", insert_pos, ch); */
-	
-	iter = ianjuta_editor_get_cell_iter (editor, insert_pos, NULL);
-	
-	if (iter_is_newline (iter, ch))
+	/* If autoindent is enabled*/
+	if (anjuta_preferences_get_int (plugin->prefs, PREF_INDENT_AUTOMATIC))
 	{
-		skip_iter_to_newline_head (iter, ch);
-		/* All newline entries means enable indenting */
-		should_auto_indent = TRUE;
-	}
-	else if (ch == '{' || ch == '}' || ch == '#')
-	{
-		/* Indent only when it's the first non-white space char in the line */
-		
-		/* Don't bother if we are inside comment or string */
-		attrib = ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL (iter),
-													NULL);
-		if (attrib != IANJUTA_EDITOR_COMMENT &&
-			attrib != IANJUTA_EDITOR_STRING)
+	
+		/* DEBUG_PRINT ("Char added at position %d: '%c'", insert_pos, ch); */	
+	
+		if (iter_is_newline (iter, ch))
 		{
-			/* Iterate backwards till the begining of the line and disable
-			 * indenting if any non-white space char is encountered
-			 */
-			
-			/* Begin by assuming it should be indented */
+			skip_iter_to_newline_head (iter, ch);
+			/* All newline entries means enable indenting */
 			should_auto_indent = TRUE;
-			
-			while (ianjuta_iterable_previous (iter, NULL))
+		}
+		else if (ch == '{' || ch == '}' || ch == '#')
+		{
+			/* Indent only when it's the first non-white space char in the line */
+		
+			/* Don't bother if we are inside string */
+			attrib = ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL (iter),
+														NULL);
+			if (attrib != IANJUTA_EDITOR_STRING)
 			{
-				ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (iter),
-												   0, NULL);
-				
-				/* DEBUG_PRINT ("Looking at char '%c'", ch); */
-				
-				/* Break on begining of line (== end of previous line) */
-				if (iter_is_newline (iter, ch))
+				/* Iterate backwards till the begining of the line and disable
+				 * indenting if any non-white space char is encountered
+				 */
+			
+				/* Begin by assuming it should be indented */
+				should_auto_indent = TRUE;
+			
+				while (ianjuta_iterable_previous (iter, NULL))
 				{
-					skip_iter_to_newline_head (iter, ch);
-					break;
-				}
-				/* If a non-white space char is encountered, disabled indenting */
-				if (!isspace (ch))
-				{
-					should_auto_indent = FALSE;
-					break;
+					ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (iter),
+													   0, NULL);
+				
+					//DEBUG_PRINT ("Looking at char '%c'", ch);
+				
+					/* Break on begining of line (== end of previous line) */
+					if (iter_is_newline (iter, ch))
+					{
+						skip_iter_to_newline_head (iter, ch);
+						break;
+					}
+					/* If a non-white space char is encountered, disabled indenting */
+					if (!isspace (ch))
+					{
+						should_auto_indent = FALSE;
+						break;
+					}
 				}
 			}
 		}
-	}
-#if 0
-	else if (ch == '\t' &&
-			 anjuta_preferences_get_int (plugin->prefs,
-										 PREF_INDENT_TAB_INDENTS))
-	{
-		/* Indent on tab enabled */
-		/* Don't bother if we are inside comment or string */
-		attrib = ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL (iter),
-													NULL);
-		if (attrib != IANJUTA_EDITOR_COMMENT &&
-			attrib != IANJUTA_EDITOR_STRING)
+		else if (ch == ':')
 		{
-			
-			/* Ensure that we remove the inserted tab */
-			ianjuta_editor_erase (editor, insert_pos, 1, NULL);
-			
 			should_auto_indent = TRUE;
 		}
-	}
-#endif
-	if (should_auto_indent)
-	{
-		gint insert_line;
-		gint line_indent;
+		if (should_auto_indent)
+		{
+			gint insert_line;
+			gint line_indent;
 		
-		ianjuta_document_begin_undo_action (IANJUTA_DOCUMENT(editor), NULL);
-		initialize_indentation_params (plugin);
-		insert_line = ianjuta_editor_get_lineno (editor, NULL);
-		line_indent = get_line_auto_indentation (plugin, editor, insert_line);
-		set_line_indentation (editor, insert_line, line_indent);
-		ianjuta_document_end_undo_action (IANJUTA_DOCUMENT(editor), NULL);
+			ianjuta_document_begin_undo_action (IANJUTA_DOCUMENT(editor), NULL);
+			initialize_indentation_params (plugin);
+			insert_line = ianjuta_editor_get_lineno (editor, NULL);
+			line_indent = get_line_auto_indentation (plugin, editor, insert_line);
+			set_line_indentation (editor, insert_line, line_indent);
+			ianjuta_document_end_undo_action (IANJUTA_DOCUMENT(editor), NULL);
+		}
+	}
+	
+	if (anjuta_preferences_get_int (plugin->prefs, PREF_BRACE_AUTOCOMPLETION))
+	{
+		if (!stack)
+			stack = g_string_new ("");
+	
+		if (ch == '[' || ch == '(')
+		{
+			ianjuta_iterable_next (iter, NULL);
+		
+			switch (ch)
+			{
+				case '[': ianjuta_editor_insert (editor, iter,
+												 "]", 1, NULL);
+						  g_string_prepend_c (stack, ']');
+						  break;
+				case '(': ianjuta_editor_insert (editor, iter,
+												 ")", 1, NULL);
+						  g_string_prepend_c (stack, ')');
+						  break;
+				default: break;
+			}
+
+			ianjuta_iterable_previous (iter, NULL);
+			ianjuta_editor_goto_position (editor, iter, NULL);
+		}
+		else if (ch == ']' || ch == ')' || ch == '"')
+		{
+			gchar *str = stack->str;
+			gchar *next_char;
+			IAnjutaIterable *end;
+
+			/* First iter*/
+			ianjuta_iterable_next (iter, NULL);
+		
+			/*
+			 * If the character is " we have to decide if we need insert
+			 * another " or we have to skip the character
+			 */
+			if (ch == '"' && *str != '"')
+			{
+				ianjuta_editor_insert (editor, iter,
+									 "\"", 1, NULL);
+				g_string_prepend_c (stack, '"');
+			
+				ianjuta_iterable_previous (iter, NULL);
+				ianjuta_editor_goto_position (editor, iter, NULL);
+				g_object_unref (iter);
+				return;
+			}
+		
+			/* End iter*/
+			end = ianjuta_iterable_clone (iter, NULL);
+			ianjuta_iterable_next (end, NULL);
+		
+			next_char = ianjuta_editor_get_text (editor, iter, end, NULL);
+		
+			if (ch == *str && ch == *next_char)
+			{
+				g_string_erase (stack, 0, 1);
+				ianjuta_editor_erase (editor, iter, end, NULL);
+			}
+			else {
+				g_string_free (stack, TRUE);
+				stack = g_string_new ("");
+			}
+			g_object_unref (end);
+		}
 	}
 	g_object_unref (iter);
 }
 
 static void
 on_editor_char_inserted_java (IAnjutaEditor *editor,
-							  gint insert_pos,
+							  IAnjutaIterable *insert_pos,
 							  gchar ch,
 							  CppJavaPlugin *plugin)
 {
@@ -1405,8 +1676,8 @@ on_auto_indent (GtkAction *action, gpointer data)
 														NULL);
 		sel_end = ianjuta_editor_selection_get_end (IANJUTA_EDITOR_SELECTION (editor),
 													NULL);
-		line_start = ianjuta_editor_cell_get_line (IANJUTA_EDITOR_CELL (sel_start), NULL);
-		line_end = ianjuta_editor_cell_get_line (IANJUTA_EDITOR_CELL (sel_end), NULL);
+		line_start = ianjuta_editor_get_line_from_position (editor, sel_start, NULL);
+		line_end = ianjuta_editor_get_line_from_position (editor, sel_end, NULL);
 		g_object_unref (sel_start);
 		g_object_unref (sel_end);
 	}
@@ -1422,7 +1693,7 @@ on_auto_indent (GtkAction *action, gpointer data)
 	{
 		line_indent = get_line_auto_indentation (lang_plugin, editor,
 												 insert_line);
-		DEBUG_PRINT ("Line indent for line %d = %d", insert_line, line_indent);
+		/* DEBUG_PRINT ("Line indent for line %d = %d", insert_line, line_indent); */
 		set_line_indentation (editor, insert_line, line_indent);
 	}
 	ianjuta_document_end_undo_action (IANJUTA_DOCUMENT(editor), NULL);
@@ -1496,7 +1767,7 @@ cpp_java_plugin_activate_plugin (AnjutaPlugin *plugin)
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	lang_plugin->action_group = 
 		anjuta_ui_add_action_group_entries (ui, "ActionGroupCppJavaAssist",
-											_("Cpp/Java Assistance"),
+											_("C++/Java Assistance"),
 											actions,
 											G_N_ELEMENTS (actions),
 											GETTEXT_PACKAGE, TRUE,
@@ -1540,14 +1811,14 @@ cpp_java_plugin_finalize (GObject *obj)
 	/* CppJavaPlugin* plugin = ANJUTA_PLUGIN_CPP_JAVA (obj); */
 
 	/* Finalization codes here */
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
 cpp_java_plugin_dispose (GObject *obj)
 {
 	/* Disposition codes */
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void
