@@ -45,6 +45,8 @@
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-plugin.h>
 #include <libanjuta/anjuta-debug.h>
+#include <libanjuta/anjuta-encodings.h>
+#include <libanjuta/anjuta-convert.h>
 #include <libanjuta/interfaces/ianjuta-editor.h>
 #include <libanjuta/interfaces/ianjuta-document.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
@@ -74,57 +76,9 @@ static SearchReplace *sr = NULL;
 
 void clear_search_replace_instance(void);
 
-
-static void
-pcre_info_free (PcreInfo *re)
-{
-	if (re)
-	{
-		if (re->re)
-			(*pcre_free)(re->re);
-		if (re->extra)
-			(*pcre_free)(re->extra);
-		if (re->ovector)
-			g_free(re->ovector);
-		g_free(re);
-	}
-}
-
-static PcreInfo *
-pcre_info_new (SearchExpression *s)
-{
-	PcreInfo *re;
-	int options = 0;
-	const char *err;
-	int err_offset;
-	int status;
-
-	g_return_val_if_fail(s && s->search_str, NULL);
-	re = g_new0(PcreInfo, 1);
-	if (s->ignore_case)
-		options |= PCRE_CASELESS;
-	if (!s->greedy)
-		options |= PCRE_UNGREEDY;
-	re->re = pcre_compile(s->search_str, options, &err, &err_offset, NULL);
-	if (NULL == re->re)
-	{
-		/* Compile failed - check error message */
-		g_warning("Regex compile failed! %s at position %d", err, err_offset);
-		pcre_info_free(re);
-		return NULL;
-	}
-	re->extra = pcre_study(re->re, 0, &err);
-	status = pcre_fullinfo(re->re, re->extra, PCRE_INFO_CAPTURECOUNT
-	  , &(re->ovec_count));
-	re->ovector = g_new0(int, 3 *(re->ovec_count + 1));
-	return re;
-}
-
-
 static void match_substr_free(MatchSubStr *ms)
 {
-	if (ms)
-		g_free(ms);
+	g_free(ms);
 }
 
 
@@ -152,6 +106,8 @@ file_buffer_free (FileBuffer *fb)
 	{
 		if (fb->path)
 			g_free(fb->path);
+		if (fb->uri)
+			g_free (fb->uri);
 		if (fb->buf)
 			g_free(fb->buf);
 		if (fb->lines)
@@ -166,6 +122,7 @@ file_buffer_new_from_te (IAnjutaEditor *te)
 {
 	FileBuffer *fb;
 	gchar* uri;
+	gchar* path;
 	
 	g_return_val_if_fail(te, NULL);
 	fb = g_new0(FileBuffer, 1);
@@ -173,19 +130,24 @@ file_buffer_new_from_te (IAnjutaEditor *te)
 	fb->te = te;
 	
 	uri = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL);
-	if (uri)
+	path = gnome_vfs_get_local_path_from_uri(uri);
+	if (path)
 	{
-		fb->path = tm_get_real_path(uri);
-		g_free (uri);
+		fb->path = tm_get_real_path(path);
+		g_free (path);
 	}
+	fb->uri = uri;
 	fb->len = ianjuta_editor_get_length(te, NULL);
-	fb->buf = ianjuta_editor_get_text(fb->te, 0, fb->len, NULL);
-	fb->pos = ianjuta_editor_get_position(fb->te, NULL);
+	fb->buf = ianjuta_editor_get_text_all (fb->te, NULL);
+	fb->pos = ianjuta_editor_get_offset(fb->te, NULL);
 	fb->line = ianjuta_editor_get_lineno(fb->te, NULL);
 	
 	return fb;
 }
 
+/* Only use the first 500 chars for validating (yes, I feel lucky...) */
+#define MAX_VALIDATE 500
+
 FileBuffer *
 file_buffer_new_from_path (const char *path, const char *buf, int len, int pos)
 {
@@ -193,6 +155,7 @@ file_buffer_new_from_path (const char *path, const char *buf, int len, int pos)
 	IAnjutaEditor *te;
 	IAnjutaDocument* doc;
 	char *real_path;
+	char *uri;
 	int i;
 	int lineno;
 
@@ -200,8 +163,10 @@ file_buffer_new_from_path (const char *path, const char *buf, int len, int pos)
 	real_path = tm_get_real_path(path);
 	
 	/* There might be an already open TextEditor with this path */
-	doc = ianjuta_document_manager_find_document_with_path (sr->docman,
-														 real_path, NULL);
+	uri = gnome_vfs_get_uri_from_local_path (real_path);
+	doc = ianjuta_document_manager_find_document_with_uri (sr->docman,
+														 uri, NULL);
+
 	if (doc && IANJUTA_IS_EDITOR (doc))
 	{
 		te = IANJUTA_EDITOR (doc);
@@ -211,6 +176,7 @@ file_buffer_new_from_path (const char *path, const char *buf, int len, int pos)
 	fb = g_new0(FileBuffer, 1);
 	fb->type = FB_FILE;
 	fb->path = real_path;
+	fb->uri = uri;
 	fb->name = strrchr(path, '/');
 	if (fb->name)
 		++ fb->name;
@@ -256,6 +222,42 @@ file_buffer_new_from_path (const char *path, const char *buf, int len, int pos)
 			}
 		}
 	}
+	if (!g_utf8_validate (fb->buf, MIN(MAX_VALIDATE, fb->len), NULL))
+	{
+		const AnjutaEncoding *encoding_used = NULL;
+		gchar* converted_text;
+		guint converted_len;
+		converted_text = anjuta_convert_to_utf8 (fb->buf, 
+												 fb->len, 
+												 &encoding_used,
+												 &converted_len, 
+												 NULL);
+		if (converted_text == NULL)
+		{
+			/* Last change, let's try 8859-15 */
+			encoding_used =  
+				anjuta_encoding_get_from_charset("ISO-8859-15");
+			
+			converted_text = anjuta_convert_to_utf8 (fb->buf,
+												  fb->len,
+												  &encoding_used,
+												  &converted_len,
+												  NULL);
+		}
+		if (converted_text == NULL)
+		{
+			/* Give up */
+			file_buffer_free(fb);
+			return NULL;
+		}
+		else
+		{
+			g_free (fb->buf);
+			fb->buf = converted_text;
+			fb->len = converted_len;
+		}
+	}
+	
 	if (pos <= 0 || pos > fb->len)
 	{
 		fb->pos = 0;
@@ -301,7 +303,11 @@ file_buffer_line_from_pos(FileBuffer *fb, int pos)
 	}
 	else if (FB_EDITOR == fb->type)
 	{
-		return ianjuta_editor_get_line_from_position(fb->te, pos, NULL);
+		IAnjutaIterable *position;
+		position = ianjuta_editor_get_position_from_offset (fb->te, pos, NULL);
+		lineno = ianjuta_editor_get_line_from_position (fb->te, position, NULL);
+		g_object_unref (position);
+		return lineno;
 	}
 	else
 		return -1;
@@ -387,18 +393,18 @@ get_project_file_list(void)
 
 
 static gboolean
-isawordchar (int c)
+isawordchar (gunichar c)
 {
-	return (isalnum(c) || '_' == c);
+	return (g_unichar_isalnum(c) || '_' == c);
 }
 
 static gboolean
-extra_match (FileBuffer *fb, SearchExpression *s, gint match_len)
+extra_match (FileBuffer *fb, gchar* begin, gchar* end, SearchExpression *s)
 {
-	gchar b, e;
+	gunichar b, e;
 	
-	b = fb->buf[fb->pos-1];
-	e = fb->buf[fb->pos+match_len];
+	b = g_utf8_get_char (g_utf8_prev_char (begin));
+	e = g_utf8_get_char (end);
 	
 	if (s->whole_line)
 		if ((fb->pos == 0 || b == '\n' || b == '\r') &&
@@ -433,55 +439,69 @@ get_next_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
 	
 	if (s->regex)
 	{
-		/* Regular expression match */
-		int options = PCRE_NOTEMPTY;
-		int status;
-		if (NULL == s->re)
+		GMatchInfo* match_info;
+		if (s->regex_info == NULL)
 		{
-			if (NULL == (s->re = pcre_info_new(s)))
-				return NULL;
-		}
-		status = pcre_exec (s->re->re, s->re->extra, fb->buf, fb->len, fb->pos,
-							options, s->re->ovector, 3 * (s->re->ovec_count + 1));
-		if (0 == status)
-		{
-			/* ovector too small - this should never happen ! */
-			g_warning("BUG ! ovector found to be too small");
-			return NULL;
-		}
-		else if (0 > status && status != PCRE_ERROR_NOMATCH)
-		{
-			/* match error - again, this should never happen */
-			g_warning("PCRE Match error");
-			return NULL;
-		}
-		else if (PCRE_ERROR_NOMATCH != status)
-		{
-			mi = g_new0(MatchInfo, 1);
-			mi->pos = s->re->ovector[0];
-			mi->len = s->re->ovector[1] - s->re->ovector[0];
-			mi->line = file_buffer_line_from_pos(fb, mi->pos);
-			if (status > 1) /* Captured subexpressions */
+			GError* error = NULL;
+			GRegexCompileFlags compile_flags = 0;
+			GRegexMatchFlags match_flags = 0;
+		
+			match_flags |= G_REGEX_MATCH_NOTEMPTY;
+			if (s->ignore_case)
 			{
-				int i;
-				MatchSubStr *ms;
-				for (i=1; i < status; ++i)
-				{
-					ms = g_new0(MatchSubStr, 1);
-					ms->start = s->re->ovector[i * 2];
-					ms->len = s->re->ovector[i * 2 + 1] - ms->start;
-					mi->subs = g_list_prepend(mi->subs, ms);
-				}
-				mi->subs = g_list_reverse(mi->subs);
+				compile_flags |= G_REGEX_CASELESS;
 			}
-			fb->pos = s->re->ovector[1];
+			if (!s->greedy)
+			{
+				compile_flags |= G_REGEX_UNGREEDY;
+			}
+			s->regex_info = g_regex_new (s->search_str, compile_flags,
+										 match_flags, &error);
+			if (error)
+			{
+				anjuta_util_dialog_error (NULL, error->message);
+				g_error_free(error);
+				s->regex_info = NULL;
+				return NULL;
+			}
+		}
+
+		g_regex_match_full (s->regex_info, fb->buf, fb->len,
+					   g_utf8_offset_to_pointer (fb->buf, fb->pos) - fb->buf, 
+					   G_REGEX_MATCH_NOTEMPTY, &match_info, NULL);
+		
+		if (g_match_info_matches (match_info))
+		{
+			gint start;
+			gint end;
+			int i;
+			mi = g_new0(MatchInfo, 1);
+			if (g_match_info_fetch_pos (match_info, 0, &start, &end))
+			{
+				DEBUG_PRINT ("Regex: %d %d", start, end);
+				mi->pos = g_utf8_pointer_to_offset (fb->buf, fb->buf + start);
+				mi->len = g_utf8_pointer_to_offset (fb->buf, fb->buf + end) - mi->pos;
+				mi->line = file_buffer_line_from_pos(fb, mi->pos);
+			}
+			for (i = 1; i < g_match_info_get_match_count(match_info); i++) /* Captured subexpressions */
+			{
+				MatchSubStr *ms;
+				ms = g_new0(MatchSubStr, 1);
+				if (g_match_info_fetch_pos (match_info, i, &start, &end))
+				{
+					ms->start = g_utf8_pointer_to_offset (fb->buf, fb->buf + start);
+					ms->len = g_utf8_pointer_to_offset (fb->buf, fb->buf + end) - ms->start;
+				}
+				mi->subs = g_list_prepend(mi->subs, ms);
+			}
+			mi->subs = g_list_reverse(mi->subs);
+			fb->pos = g_utf8_pointer_to_offset (fb->buf, fb->buf + end);
 		}
 	}
 	else
 	{
 		/* Simple string search - this needs to be performance-tuned */
 		gboolean found;
-		gchar lc;
 		gint match_len;
 
 		match_len = strlen (s->search_str);
@@ -494,35 +514,45 @@ get_next_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
 			/* Backward matching. */
 			if (s->ignore_case)
 			{
-				/* FIXME support encodings with > 1 byte per char */
-				lc = tolower (s->search_str[0]);
-				for (; fb->pos != -1; --fb->pos)
+				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
+				gint len = g_utf8_strlen (s->search_str, -1);
+				gchar* search_caseless = g_utf8_casefold (s->search_str, len);
+				for (; fb->pos >= len; --fb->pos)
 				{
-					if (lc == tolower(fb->buf[fb->pos]))
+					gchar* current_caseless = g_utf8_casefold (current, len);
+					if (g_utf8_collate (current_caseless, search_caseless) == 0 &&
+						extra_match (fb, current, current + strlen (search_caseless),
+									 s))
 					{
-						if (0 == g_strncasecmp(s->search_str, fb->buf + fb->pos,
-						    match_len) && extra_match (fb, s, match_len))
-						{
-							found = TRUE;
-							break;
-						}
+						found = TRUE;
+						g_free (current_caseless);
+						break;
 					}
+					else
+						current = g_utf8_prev_char (current);
 				}
+				g_free (search_caseless);
 			}
 			else
 			{
-				for (; fb->pos != -1; --fb->pos)
+				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
+				gint len = g_utf8_strlen (s->search_str, -1);
+				gchar* search_key = g_utf8_collate_key (s->search_str, len);
+				for (; fb->pos >= len; --fb->pos)
 				{
-					if (s->search_str[0] == fb->buf[fb->pos])
+					gchar* current_key = g_utf8_collate_key (current, len);
+					if (g_str_equal (current_key, search_key) &&
+						extra_match (fb, current, current + strlen (s->search_str),
+									 s))
 					{
-						if (0 == strncmp(s->search_str, fb->buf + fb->pos,
-						  	match_len) && extra_match (fb, s, match_len))
-						{
-							found = TRUE;
-							break;
-						}
+						found = TRUE;
+						g_free (current_key);
+						break;
 					}
+					else
+						current = g_utf8_prev_char (current);
 				}
+				g_free (search_key);
 			}
 		}
 		else
@@ -530,35 +560,47 @@ get_next_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
 			/* Forward match */
 			if (s->ignore_case)
 			{
-				/* FIXME support encodings with > 1 byte per char */
-				lc = tolower (s->search_str[0]);
-				for (; fb->pos < fb->len; ++fb->pos)
+				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
+				gint len = g_utf8_strlen (s->search_str, -1);
+				gchar* search_caseless = g_utf8_casefold (s->search_str, len);
+				gint buf_len = g_utf8_strlen (fb->buf, fb->len);
+				for (; fb->pos < buf_len; ++fb->pos)
 				{
-					if (lc == tolower(fb->buf[fb->pos]))
+					gchar* current_caseless = g_utf8_casefold (current, len);
+					if (g_utf8_collate (current_caseless, search_caseless) == 0 &&
+						extra_match (fb, current, current + strlen (search_caseless),
+									 s))
 					{
-						if (0 == g_strncasecmp(s->search_str, fb->buf + fb->pos,
-						    match_len) && extra_match (fb, s, match_len))
-						{
-							found = TRUE;
-							break;
-						}
+						found = TRUE;
+						g_free (current_caseless);
+						break;
 					}
+					else
+						current = g_utf8_next_char (current);
 				}
+				g_free (search_caseless);
 			}
 			else
 			{
-				for (; fb->pos < fb->len; ++fb->pos)
+				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
+				gint len = g_utf8_strlen (s->search_str, -1);
+				gint buf_len = g_utf8_strlen (fb->buf, fb->len);
+				gchar* search_key = g_utf8_collate_key (s->search_str, len);
+				for (; fb->pos < buf_len; ++fb->pos)
 				{
-					if (s->search_str[0] == fb->buf[fb->pos])
+					gchar* current_key = g_utf8_collate_key (current, len);
+					if (g_str_equal (current_key, search_key) &&
+						extra_match (fb, current, current + strlen (s->search_str),
+									 s))
 					{
-						if (0 == strncmp(s->search_str, fb->buf + fb->pos,
-						    match_len) && extra_match (fb, s, match_len))
-						{
-							found = TRUE;
-							break;
-						}
+						found = TRUE;
+						g_free (current_key);
+						break;
 					}
+					else
+						current = g_utf8_next_char (current);
 				}
+				g_free (search_key);
 			}
 		}
 		if (found)
@@ -638,7 +680,7 @@ create_search_entries (Search *s)
 					}
 					else
 					{
-						se->start_pos = ianjuta_editor_get_position (se->te, NULL);
+						se->start_pos = ianjuta_editor_get_offset (se->te, NULL);
 					}
 					se->end_pos = -1;	/* not actually used when backward searching */
 				}
@@ -703,8 +745,8 @@ create_search_entries (Search *s)
 				if (s->range.type != SR_SELECTION)
 				{
 					IAnjutaIterable *start, *end;
-					start = ianjuta_editor_get_cell_iter (se->te, selstart, NULL);
-					end = ianjuta_editor_get_cell_iter (se->te, selend, NULL);
+					start = ianjuta_editor_get_position_from_offset (se->te, selstart, NULL);
+					end = ianjuta_editor_get_position_from_offset (se->te, selend, NULL);
 					ianjuta_editor_selection_set(IANJUTA_EDITOR_SELECTION (se->te), 
 				                                 start, end, NULL);	
 					g_object_unref (start);
@@ -836,8 +878,11 @@ void
 clear_search_replace_instance(void)
 {
 	g_free (sr->search.expr.search_str);
-	g_free (sr->search.expr.re);
-	FREE_FN(pcre_info_free, sr->search.expr.re);
+	if (sr->search.expr.regex_info)
+	{
+		g_regex_unref (sr->search.expr.regex_info);
+		sr->search.expr.regex_info = NULL;
+	}
 	if (SR_FILES == sr->search.range.type)
 	{
 		FREE_FN(anjuta_util_glist_strings_free, sr->search.range.files.match_files);
@@ -850,17 +895,15 @@ clear_search_replace_instance(void)
 	FREE_FN(anjuta_util_glist_strings_free, sr->replace.expr_history);
 }
 
-void
-clear_pcre(void)
-{
-	FREE_FN(pcre_info_free, sr->search.expr.re);
-}
-
 SearchReplace *
 create_search_replace_instance(IAnjutaDocumentManager *docman)
 {
-	if (NULL == sr) /* Create a new SearchReplace instance */
+	/* Create a new SearchReplace instance */
+	if (NULL == sr)
+	{
 		sr = g_new0(SearchReplace, 1);
+		sr->search.expr.regex_info = NULL;
+	}
 	else
 		clear_search_replace_instance ();
 	if (docman)
