@@ -27,7 +27,7 @@
  * list of people on the anjuta Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: anjuta-view.c 3766 2008-03-11 14:05:05Z jhs $ 
+ * $Id: anjuta-view.c 3918 2008-05-06 11:15:58Z jhs $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,9 +43,9 @@
 #include <glib.h>
 
 #include <libanjuta/anjuta-debug.h>
+#include <libanjuta/anjuta-encodings.h>
 
 #include "anjuta-view.h"
-#include "anjuta-encodings.h"
 #include "sourceview.h"
 #include "sourceview-private.h"
 #include "anjuta-marshal.h"
@@ -528,137 +528,6 @@ anjuta_view_scroll_to_cursor (AnjutaView *view)
 	view->priv->scroll_idle = g_idle_add ((GSourceFunc) scroll_to_cursor_real, view);
 }
 
-/* assign a unique name */
-static G_CONST_RETURN gchar *
-get_widget_name (GtkWidget *w)
-{
-	const gchar *name;	
-
-	name = gtk_widget_get_name (w);
-	g_return_val_if_fail (name != NULL, NULL);
-
-	if (strcmp (name, g_type_name (GTK_WIDGET_TYPE (w))) == 0)
-	{
-		static guint d = 0;
-		gchar *n;
-
-		n = g_strdup_printf ("%s_%u_%u", name, d, (guint) g_random_int);
-		d++;
-
-		gtk_widget_set_name (w, n);
-		g_free (n);
-
-		name = gtk_widget_get_name (w);
-	}
-
-	return name;
-}
-
-/* There is no clean way to set the cursor-color, so we are stuck
- * with the following hack: set the name of each widget and parse
- * a gtkrc string.
- */
-static void 
-modify_cursor_color (GtkWidget *textview, 
-		     GdkColor  *color)
-{
-	static const char cursor_color_rc[] =
-		"style \"svs-cc\"\n"
-		"{\n"
-			"GtkSourceView::cursor-color=\"#%04x%04x%04x\"\n"
-		"}\n"
-		"widget \"*.%s\" style : application \"svs-cc\"\n";
-
-	const gchar *name;
-	gchar *rc_temp;
-
-	name = get_widget_name (textview);
-	g_return_if_fail (name != NULL);
-
-	if (color != NULL)
-	{
-		rc_temp = g_strdup_printf (cursor_color_rc,
-					   color->red, 
-					   color->green, 
-					   color->blue,
-					   name);
-	}
-	else
-	{
-		GtkRcStyle *rc_style;
-
- 		rc_style = gtk_widget_get_modifier_style (textview);
-
-		rc_temp = g_strdup_printf (cursor_color_rc,
-					   rc_style->text [GTK_STATE_NORMAL].red,
-					   rc_style->text [GTK_STATE_NORMAL].green,
-					   rc_style->text [GTK_STATE_NORMAL].blue,
-					   name);
-	}
-
-	gtk_rc_parse_string (rc_temp);
-	gtk_widget_reset_rc_styles (textview);
-
-	g_free (rc_temp);
-}
-
-void
-anjuta_view_set_colors (AnjutaView *view,
-		       gboolean   def,
-		       GdkColor  *backgroud,
-		       GdkColor  *text,
-		       GdkColor  *selection,
-		       GdkColor  *sel_text)
-{
-
-	g_return_if_fail (ANJUTA_IS_VIEW (view));
-
-	/* just a bit of paranoia */
-	gtk_widget_ensure_style (GTK_WIDGET (view));
-
-	if (!def)
-	{
-		if (backgroud != NULL)
-			gtk_widget_modify_base (GTK_WIDGET (view), 
-						GTK_STATE_NORMAL, backgroud);
-
-		if (selection != NULL)
-		{
-			gtk_widget_modify_base (GTK_WIDGET (view), 
-						GTK_STATE_SELECTED, selection);
-			gtk_widget_modify_base (GTK_WIDGET (view), 
-						GTK_STATE_ACTIVE, selection);
-		}
-
-		if (sel_text != NULL)
-		{
-			gtk_widget_modify_text (GTK_WIDGET (view), 
-						GTK_STATE_SELECTED, sel_text);		
-			gtk_widget_modify_text (GTK_WIDGET (view), 
-						GTK_STATE_ACTIVE, sel_text);		
-		}
-
-		if (text != NULL)
-		{
-			gtk_widget_modify_text (GTK_WIDGET (view), 
-						GTK_STATE_NORMAL, text);
-			modify_cursor_color (GTK_WIDGET (view), text);
-		}
-	}
-	else
-	{
-		GtkRcStyle *rc_style;
-
-		rc_style = gtk_widget_get_modifier_style (GTK_WIDGET (view));
-
-		rc_style->color_flags [GTK_STATE_NORMAL] = 0;
-		rc_style->color_flags [GTK_STATE_SELECTED] = 0;
-		rc_style->color_flags [GTK_STATE_ACTIVE] = 0;
-
-		gtk_widget_modify_style (GTK_WIDGET (view), rc_style);
-	}
-}
-
 void
 anjuta_view_set_font (AnjutaView   *view, 
 		     gboolean     def, 
@@ -780,6 +649,24 @@ anjuta_view_button_press_event	(GtkWidget *widget, GdkEventButton *event)
 	{
 		case 3: /* Right Button */
 		{
+			GtkTextBuffer* buffer = GTK_TEXT_BUFFER (view->priv->sv->priv->document);
+			if (!gtk_text_buffer_get_has_selection (buffer))
+			{
+				/* Move cursor to set breakpoints at correct line (#530689) */
+				GtkTextIter iter;
+				gint buffer_x, buffer_y;
+				GtkTextWindowType type =  gtk_text_view_get_window_type (GTK_TEXT_VIEW (view),
+																																 event->window);
+				gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (view),
+																							 type,
+																							 event->x,
+																							 event->y,
+																							 &buffer_x,
+																							 &buffer_y);
+				gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (view),
+																						&iter, buffer_x, buffer_y);
+				gtk_text_buffer_place_cursor (buffer, &iter);
+			}												 
 			gtk_menu_popup (GTK_MENU (view->priv->popup), NULL, NULL, NULL, NULL, 
                   event->button, event->time);
 			return TRUE;
