@@ -25,6 +25,7 @@
 
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/anjuta-preferences.h>
+#include <libanjuta/anjuta-encodings.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
 #include <libanjuta/interfaces/ianjuta-file-savable.h>
 #include <libanjuta/interfaces/ianjuta-markable.h>
@@ -53,7 +54,6 @@
 #include <gtksourceview/gtksourceiter.h>
 
 #include "config.h"
-#include "anjuta-encodings.h"
 #include "anjuta-document.h"
 #include "anjuta-view.h"
 
@@ -80,6 +80,7 @@ static void sourceview_class_init(SourceviewClass *klass);
 static void sourceview_instance_init(Sourceview *sv);
 static void sourceview_finalize(GObject *object);
 static void sourceview_dispose(GObject *object);
+
 static GObjectClass *parent_class = NULL;
 
 #if HAVE_TOOLTIP_API
@@ -426,7 +427,7 @@ static void
 sourceview_adjustment_changed(GtkAdjustment* ad, Sourceview* sv)
 {
 	/* Hide assistance windows when scrolling vertically */
-	
+
 	if (sv->priv->assist_win)
 		gtk_widget_destroy (GTK_WIDGET (sv->priv->assist_win));
 	if (sv->priv->assist_tip)
@@ -443,7 +444,7 @@ static void
 sourceview_class_init(SourceviewClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
-
+	
 	parent_class = g_type_class_peek_parent(klass);
 	object_class->dispose = sourceview_dispose;
 	object_class->finalize = sourceview_finalize;
