@@ -103,7 +103,6 @@ sdb_view_get_iter_from_row_ref (SymbolDBView *dbv, GtkTreeRowReference *row_ref,
 	if (row_ref == NULL) 
 	{
 		/* no node displayed found */
-		DEBUG_PRINT ("sdb_view_get_iter_from_row_ref (): row_ref == NULL");
 		return FALSE;
 	}
 			
@@ -468,7 +467,7 @@ prepare_for_adding (SymbolDBView *dbv, gint parent_symbol_id,
 	
 	g_return_if_fail (dbv != NULL);	
 	priv = dbv->priv;
-
+	
 	/* add to root if parent_symbol_id is <= 0 */
 	if (parent_symbol_id <= 0)
 	{
@@ -525,9 +524,6 @@ prepare_for_adding (SymbolDBView *dbv, gint parent_symbol_id,
 
 		if (curr_tree_row_ref == NULL)
 		{
-			g_warning ("prepare_for_adding (): row_ref == NULL. symbol_name %s "
-					   "symbol_id %d kind %s parent_symbol_id %d", 
-					   symbol_name, symbol_id, kind, parent_symbol_id);
 			return;
 		}		
 		
@@ -767,7 +763,6 @@ on_symbol_removed (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
 	row_ref = g_tree_lookup (priv->nodes_displayed, (gpointer)symbol_id);
 	if (sdb_view_get_iter_from_row_ref (dbv, row_ref, &iter) == FALSE)
 	{
-		g_warning ("on_symbol_removed (): iter was not set ?![%d]", symbol_id);
 		return;
 	}
 	
@@ -821,7 +816,6 @@ sdb_view_row_expanded_idle_destroy (gpointer data)
 	NodeIdleExpand *node_expand;
 	SymbolDBView *dbv;
 	SymbolDBEngine *dbe;
-	SymbolDBEngineIterator *iterator;
 	
 	g_return_if_fail (data != NULL);
 	node_expand = data;	
@@ -892,7 +886,6 @@ sdb_view_row_expanded_idle (gpointer data)
 										symbol_name, curr_symbol_id);
 	if (curr_tree_row_ref == NULL)
 	{
-		g_warning ("sdb_view_global_row_expanded (): row_ref == NULL");
 		return symbol_db_engine_iterator_move_next (iterator);
 	}		
 		
@@ -978,13 +971,11 @@ sdb_view_namespace_row_expanded (SymbolDBView *dbv, SymbolDBEngine *dbe,
 	/* get results from database */
 	iterator = symbol_db_engine_get_scope_members_by_symbol_id_filtered (dbe, 
 									expanded_symbol_id, 
-									-1,
-									-1,
-									SYMINFO_SIMPLE|
-									SYMINFO_KIND|
-									SYMINFO_ACCESS,
 									filter_array,
-									TRUE
+									TRUE,
+									-1,
+									-1,
+									SYMINFO_SIMPLE| SYMINFO_KIND| SYMINFO_ACCESS
 									);
 
 	g_ptr_array_free (filter_array, TRUE);
@@ -1077,7 +1068,7 @@ sdb_view_global_row_expanded (SymbolDBView *dbv, SymbolDBEngine *dbe,
 						 (gpointer) node_expand,
 						 (GDestroyNotify) sdb_view_row_expanded_idle_destroy);
 		
-		/* insert the idle_id into a g_tree */
+		/* insert the idle_id into a g_tree for (eventually) a later retrieval */
 		DEBUG_PRINT ("Inserting into g_tree expanded_symbol_id %d and idle_id %d", 
 					 expanded_symbol_id, idle_id);
 		g_tree_insert (priv->expanding_gfunc_ids, (gpointer)expanded_symbol_id, 
@@ -1130,13 +1121,13 @@ sdb_view_vars_row_expanded (SymbolDBView *dbv, SymbolDBEngine *dbe,
 	{
 		iterator = symbol_db_engine_get_scope_members_by_symbol_id_filtered (dbe, 
 									positive_symbol_expanded, 
+									filter_array,
+									FALSE,
 									-1,
 									-1,
 									SYMINFO_SIMPLE|
 									SYMINFO_KIND|
-									SYMINFO_ACCESS,
-									filter_array,
-									FALSE
+									SYMINFO_ACCESS
 									);
 	}
 	
@@ -1373,11 +1364,14 @@ symbol_db_view_row_collapsed (SymbolDBView *dbv, SymbolDBEngine *dbe,
 }
 
 static GtkTreeStore *
-sdb_view_locals_create_new_store ()
+sdb_view_create_new_store ()
 {
 	GtkTreeStore *store;
 	store = gtk_tree_store_new (COLUMN_MAX, GDK_TYPE_PIXBUF,
-				    G_TYPE_STRING, G_TYPE_INT);	
+				    G_TYPE_STRING, G_TYPE_INT);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
+										  COLUMN_NAME,
+										  GTK_SORT_ASCENDING);
 	return store;
 }
 
@@ -1567,6 +1561,7 @@ sdb_view_load_symbol_pixbufs ()
 	
 	/* special icon */
 	CREATE_SYM_ICON ("othersvars",   "Icons.16x16.Event");
+	CREATE_SYM_ICON ("globalglobal", "Icons.16x16.Event");
 }
 
 /**
@@ -1672,6 +1667,7 @@ sdb_view_build_and_display_base_tree (SymbolDBView *dbv, SymbolDBEngine *dbe)
 	GPtrArray *filter_array;
 	GtkTreeRowReference *global_tree_row_ref;
 	GtkTreeIter global_child_iter;
+	const GdkPixbuf *global_pixbuf;
 	
 	g_return_if_fail (dbv != NULL);
 	
@@ -1724,8 +1720,7 @@ sdb_view_build_and_display_base_tree (SymbolDBView *dbv, SymbolDBEngine *dbe)
 												symbol_name, curr_symbol_id);
 			if (curr_tree_row_ref == NULL)
 			{
-				g_warning ("sdb_view_build_and_display_base_tree (): row_ref == NULL");
-				continue;
+				continue;				
 			}		
 		
 			/* we'll fake the gpointer to store an int */
@@ -1751,12 +1746,13 @@ sdb_view_build_and_display_base_tree (SymbolDBView *dbv, SymbolDBEngine *dbe)
 	/*
 	 * Good. Add a 'Global' node to the store. 
 	 */
-	global_tree_row_ref = do_add_root_symbol_to_view (dbv, NULL, 
+	global_pixbuf = symbol_db_view_get_pixbuf ("global", "global");
+	
+	global_tree_row_ref = do_add_root_symbol_to_view (dbv, global_pixbuf, 
 											"Global", ROOT_GLOBAL);
 		
 	if (global_tree_row_ref == NULL)
 	{
-		g_warning ("sdb_view_build_and_display_base_tree (): row_ref == NULL");
 		return;
 	}		
 	g_tree_insert (priv->nodes_displayed, (gpointer)ROOT_GLOBAL, 
@@ -1783,9 +1779,10 @@ symbol_db_view_recv_signals_from_engine (SymbolDBView *dbv, SymbolDBEngine *dbe,
 
 	g_return_if_fail (dbv != NULL);
 	priv = dbv->priv;
-	
+				
 	if (enable_status == TRUE) 
 	{
+		gtk_widget_set_sensitive (GTK_WIDGET (dbv), TRUE);
 		/* connect some signals */
 		if (priv->insert_handler <= 0) 
 		{
@@ -1807,6 +1804,8 @@ symbol_db_view_recv_signals_from_engine (SymbolDBView *dbv, SymbolDBEngine *dbe,
 	}
 	else		/* disconnect them, if they were ever connected before */
 	{
+		gtk_widget_set_sensitive (GTK_WIDGET (dbv), FALSE);
+		
 		if (priv->insert_handler >= 0) 
 		{
 			g_signal_handler_disconnect (G_OBJECT (dbe), priv->insert_handler);
@@ -1840,7 +1839,7 @@ symbol_db_view_open (SymbolDBView *dbv, SymbolDBEngine *dbe)
 	DEBUG_PRINT ("symbol_db_view_open ()");
 	symbol_db_view_clear_cache (dbv);
 	
-	store = sdb_view_locals_create_new_store ();
+	store = sdb_view_create_new_store ();
 	gtk_tree_view_set_model (GTK_TREE_VIEW (dbv), GTK_TREE_MODEL (store));
 	
 	priv->nodes_displayed = g_tree_new_full ((GCompareDataFunc)&gtree_compare_func, 
